
echo ""
echo "       Dcontour.sh  /  ncl-metno 1.2"
#echo "          Date:     11 April, 2006"
#echo "          Contact:  a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-11  included option to use 2d fields for lon's & lat's;
#             implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots;
#             now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>> -> for 2D fields:"
 echo ">>>  Dcontour.sh <option> <file1> <file2> <var1> <var2>"
 echo ">>> -> for 3D fields:"
 echo ">>>  Dcontour.sh <option> <file1> <file2> <var1> <var2> [<d3node1> <d3node2>]"
 echo ">>> -> for 4D fields:"
 echo ">>>  Dcontour.sh <option> <file1> <file2> <var1> <var2> [<d3node1> <d3node2> (<d4node1> <d4node2>)]"
 echo ">>> "
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  [234]dMap - [234]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[234]d[mM]ap will be displayed on a geogr. map"
 echo ">>>    <file1>     name of netcdf file w/<var1>"
 echo ">>>    <file2>     name of netcdf file w/<var2>,"
 echo ">>>                 you may type '.' if <file1> & <file2> are the same"
 echo ">>>    <var1>      name of requested variable on <file1> (case sensitive)"
 echo ">>>    <var2>      name of requested variable on <file2> (case sensitive)"
 echo ">>>                 you may type '.' if <var1> & <var2> are the same"
 echo ">>>    <d3node1>, <d3node2>"
 echo ">>>                node no.s of third dimension for <var1> and <var2>,"
 echo "                    respectively"
 echo ">>>                you may type '.' for <d3node2> if <d3node1>"
 echo ">>>                 and <d3node2> are the same"
 echo ">>>                if <option> is one of 2d, 2dmap and a fourth"
 echo ">>>                  argument is present, or if <d3node1> is negative,"
 echo ">>>                  this will be interpreted as a flag that will cause"
 echo ">>>                  the ncl script to remain (see examples below)"
 echo ">>>    <d4node1>, <d4node2>"
 echo ">>>                node no.s of fourth dimension for <var1> and <var2>,"
 echo "                    respectively"
 echo ">>>                you may type '.' for <d4node2> if <d4node1>"
 echo ">>>                 and <d4node2> are the same"
 echo ">>> "
 echo ">>>  Special case:"
 echo ">>>   If <option> is one of [234]dMap, the name of the 2d longitude and"
 echo ">>>    latitude variable may be specified on the command line, e.g. for 3D fields:"
 echo ">>>   Dcontour.sh <option> <lonname> <latname> <file1> <file2> <var1> <var2> [<d3node1> <d3node2>]"
 echo ">>>    (Alternatively, if these names are not 'lon' or 'lat', 'userdef.ncl' may"
 echo ">>>     be edited when option is one of [234]dMap.)"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'Dcontour.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * zooming"
 echo ">>>   * map projection (lon-lat grids only)"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   * coastline detail level"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Examples:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  Dcontour.sh 4dmap hydro1.nc hydro2.nc temp T 1 1 10 10"
 echo ">>>    will produce a depiction on a lon-lat grid w/ a map,"
 echo ">>>     of the difference at the first vertical level and"
 echo ">>>     the tenth time step of the variables 'temp' and 'T',"
 echo ">>>     between these fields on 'hydro1.nc' and 'hydro2.nc'"
 echo ">>>     (positive where  hydro1.nc->temp .gt. hydro2.nc->T;"
 echo ">>>     when the third and fourth dimensions for both variables"
 echo ">>>     are vertical level and time, respectively"
 echo ">>>  Dcontour.sh 4dmap hydro1.nc . salt . 4 3 10 ."
 echo ">>>    will produce a depiction on a lon-lat grid w/ a map,"
 echo ">>>     of the difference at the tenth time step on 'hydro1.nc'"
 echo ">>>     between 'salt' at the fourth and third vertical level"
 echo ">>>     (positive where  salt(,,4,) .gt. salt(,,3,);"
 echo ">>>     when the third and fourth dimensions are vertical level"
 echo ">>>     and time, respectively"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$5" = "" ]
then

 echo ">>> "
 echo ">>> Syntax error. Correct syntax:"
 echo ">>> -> for 2D fields:"
 echo ">>>  Dcontour.sh <option> <file1> <file2> <var1> <var2>"
 echo ">>> -> for 3D fields:"
 echo ">>>  Dcontour.sh <option> <file1> <file2> <var1> <var2> [<d3node1> <d3node2>]"
 echo ">>> -> for 4D fields:"
 echo ">>>  Dcontour.sh <option> <file1> <file2> <var1> <var2> [<d3node1> <d3node2> (<d4node1> <d4node2>)]"
 echo ">>> "
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  [234]dMap - [234]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[234]d[mM]ap will be displayed on a geogr. map"
 echo ">>> e.g. (for hydro1.nc->temp - hydro2.nc->T)"
 echo ">>>  Dcontour.sh 4dmap hydro1.nc hydro2.nc temp T 1 1 10 10"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   Dcontour.sh --help"
 echo ">>>  for more details, and examples."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".Dcontour.ncl"
test -r nclfile && rm nclfile

option=$1
ncfile1=$2
if [ "$option" = "4dMap" ] || [ "$option" = "3dMap" ] || [ "$option" = "2dMap" ]
then
 dotpos=`expr index "$ncfile1" .`
 if [ $dotpos -eq 0 ]
 then
  lonname=$ncfile1
  shift
  latname=$2
  shift
  ncfile1=$2
 else
  lonname=""
  latname=""
 fi
else
 lonname=""
 latname=""
fi
if [ "$3" = "." ]
then
 ncfile2=$ncfile1
else
 ncfile2=$3
fi
ncvar1=$4
if [ "$5" = "." ]
then
 ncvar2=$ncvar1
else
 ncvar2=$5
fi


if [ "$option" = "4dMap" ] || [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$9" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'Dcontour.sh --help'   for details"
  exit
 fi

 pd3=$6
 if [ $pd3 -gt 0 ]
 then
   d3a=$pd3
 else
   d3a=$((0-pd3))
 fi

 if [ "$7" = "." ]
 then
  d3b=$d3a
 else
  d3b=$7
 fi

 pd4=$8
 if [ $pd4 -gt 0 ]
 then
   d4a=$pd4
 else
   d4a=$((0-pd4))
 fi

 if [ "$9" = "." ]
 then
  d4b=$d4a
 else
  d4b=$9
 fi

 title='":F"+fontno+":'$ncvar1'('$d3a','$d4a')-'$ncvar2'('$d3b','$d4b')"'

 inc3="yes"
 inc4="yes"

elif [ "$option" = "3dMap" ] || [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 if [ "$7" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'Dcontour.sh --help'   for details"
  exit
 fi

 pd3=$6
 if [ $pd3 -gt 0 ]
 then
   d3a=$pd3
   pd4="1"
 else
   d3a=$((0-$pd3))
   pd4="0"
 fi

 if [ "$7" = "." ]
 then
  d3b=$d3a
 else
  d3b=$7
 fi

 title='":F"+fontno+":'$ncvar1'('$d3a')-'$ncvar2'('$d3b')"'

 pd4=$pd3
 inc3="yes"
 inc4="no"

elif [ "$option" = "2dMap" ] || [ "$option" = "2dmap" ] || [ "$option" = "2d" ]
then

 title='":F"+fontno+":'$ncvar1'-'$ncvar2'"'

 if [ "$6" = "" ]
 then
   pd3="1"
   pd4="1"
 else
   pd3="0"
   pd4="0"
 fi
 inc3="no"
 inc4="no"

else

 echo ">>> Unrecognized <option>, type  'Dcontour.sh --help'  for details"
 exit

fi

if   [ "$option" = "2dMap" ] || [ "$option" = "3dMap" ] || [ "$option" = "4dMap" ]
then
 prjctntype="LL"
elif [ "$option" = "2dmap" ] || [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc4" = "no" ] && [ "$inc3" = "no" ]
then
 dim0string='(:,:)'
 dim1string='(y1:y2,x1:x2)'
 dim2string='(y1:y2,x1:x2)'
elif [ "$inc4" = "no" ]
then
 dim0string='(d3a0,:,:)'
 dim1string='(d3a0,y1:y2,x1:x2)'
 dim2string='(d3b0,y1:y2,x1:x2)'
else
 dim0string='(d4a0,d3a0,:,:)'
 dim1string='(d4a0,d3a0,y1:y2,x1:x2)'
 dim2string='(d4b0,d3b0,y1:y2,x1:x2)'
fi


cat $ncdir/head.ncl                                         > $nclfile

echo 'ncfile  = addfile("'$ncfile1'","r")'                 >> $nclfile
echo 'ncfile2 = addfile("'$ncfile2'","r")'                 >> $nclfile

if [ "$inc3" = "yes" ]
then
 echo 'd3a0 = '$d3a'-1'                                    >> $nclfile
 echo 'd3b0 = '$d3b'-1'                                    >> $nclfile
fi
if [ "$inc4" = "yes" ]
then
 echo 'd4a0 = '$d4a'-1'                                    >> $nclfile
 echo 'd4b0 = '$d4b'-1'                                    >> $nclfile
fi

echo ''                                                    >> $nclfile
echo 'ncvar0 = ncfile->'$ncvar1$dim0string                 >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                              >> $nclfile
echo 'nx  = nxy(1)'                                        >> $nclfile
echo 'ny  = nxy(0)'                                        >> $nclfile

if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                              >> $nclfile
cat $ncdir/colormap.ncl                                    >> $nclfile

if   [ "$prjctntype" = "xy" ]
then
 cat $ncdir/limits.ncl                                     >> $nclfile
else
 if [ "$lonname" = "" ]
 then
  llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  lonname=${llstring:ca:cn}
 fi
 if [ "$latname" = "" ]
 then
  llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  latname=${llstring:ca:cn}
 fi
 cat $ncdir/lllimits.ncl                                   >> $nclfile
fi

echo ''                                                    >> $nclfile
echo 'ncvar  = ncfile->'$ncvar1$dim1string                 >> $nclfile
echo 'ncvar2 = ncfile2->'$ncvar2$dim2string                >> $nclfile

echo 'if(isatt(ncvar,"_FillValue"))'                       >> $nclfile
echo ' ncvar@_FillValue = ncfile->'$ncvar1'@_FillValue'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(ncvar,"missing_value"))'                   >> $nclfile
echo '  ncvar@_FillValue = ncfile->'$ncvar1'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(ncvar2,"_FillValue"))'                      >> $nclfile
echo ' ncvar2@_FillValue = ncfile->'$ncvar2'@_FillValue'   >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(ncvar2,"missing_value"))'                  >> $nclfile
echo '  ncvar2@_FillValue = ncfile->'$ncvar2'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"add_offset"))'                       >> $nclfile
echo ' addoff1 = ncfile->'$ncvar1'@add_offset'             >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoff1 = 0.'                                       >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(ncvar2,"add_offset"))'                      >> $nclfile
echo ' addoff2 = ncfile2->'$ncvar2'@add_offset'            >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoff2 = 0.'                                       >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"scale_factor"))'                     >> $nclfile
echo ' scale1 = ncfile->'$ncvar1'@scale_factor'            >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scale1 = 1.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(ncvar2,"scale_factor"))'                    >> $nclfile
echo ' scale2 = ncfile2->'$ncvar2'@scale_factor'           >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scale2 = 1.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile

if   [ "$prjctntype" = "LL" ]
then
 cat $ncdir/llsize.ncl                                     >> $nclfile
 cat $ncdir/contour.ncl                                    >> $nclfile
 echo "if (isfilevarcoord(ncfile,\""$ncvar1"\",\""$latname"\") .or. isfilevarcoord(ncfile,\""$ncvar1"\",\""$lonname"\")) then" >> $nclfile
 echo " print(\"..."$lonname" and/or "$latname" are not 2D fields\")" >> $nclfile
 echo " print(\"   which they must be for the Map options, but\")" >> $nclfile
 echo " print(\"   one or both are (1D) variable coordinate(s)\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"                >> $nclfile
 echo " exit"                                              >> $nclfile
 echo "else"                                               >> $nclfile
 echo " if (isfilevarcoord(ncfile2,\""$ncvar2"\",\""$latname"\") .or. isfilevarcoord(ncfile2,\""$ncvar2"\",\""$lonname"\")) then" >> $nclfile
 echo "  print(\"..."$lonname" and/or "$latname" are not 2D fields\")" >> $nclfile
 echo "  print(\"   which they must be for the Map options, but\")" >> $nclfile
 echo "  print(\"   one or both are (1D) variable coordinate(s)\")" >> $nclfile
 echo "  system(\"/bin/rm -f ncvar_e.eps\")"               >> $nclfile
 echo "  exit"                                             >> $nclfile
 echo " else"                                              >> $nclfile
 echo "  if (isfilevar(ncfile,\""$latname"\") .and. isfilevar(ncfile,\""$lonname"\")) then" >> $nclfile
 echo "   lon2d  = ncfile->"$lonname"(y1:y2,x1:x2)"        >> $nclfile
 echo "   lat2d  = ncfile->"$latname"(y1:y2,x1:x2)"        >> $nclfile
 cat $ncdir/mod_lon.ncl                                    >> $nclfile
 echo "  else"                                             >> $nclfile
 echo "   print(\"...could not find a proper set of map variables,\")" >> $nclfile
 echo "   print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo "   system(\"/bin/rm -f ncvar_e.eps\")"              >> $nclfile
 echo "   exit"                                            >> $nclfile
 echo "  end if"                                           >> $nclfile
 echo " end if"                                            >> $nclfile
 echo "end if"                                             >> $nclfile
 cat $ncdir/LLplot.ncl                                     >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 cat $ncdir/llsize.ncl                                     >> $nclfile
 cat $ncdir/contour.ncl                                    >> $nclfile
 echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/))) .and. all(iscoord(ncvar2,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
 echo " lon_min  = ncvar&"$lonname"(0)"                    >> $nclfile
 echo " lon_max  = ncvar&"$lonname"(x2-x1)"                >> $nclfile
 echo " lat_min  = ncvar&"$latname"(0)"                    >> $nclfile
 echo " lat_max  = ncvar&"$latname"(y2-y1)"                >> $nclfile
 echo "else"                                               >> $nclfile
 echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
 echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"                >> $nclfile
 echo " exit"                                              >> $nclfile
 echo "end if"                                             >> $nclfile
 cat $ncdir/llplot.ncl                                     >> $nclfile
else
 cat $ncdir/size.ncl                                       >> $nclfile
 cat $ncdir/contour.ncl                                    >> $nclfile
 cat $ncdir/ticklabels.ncl                                 >> $nclfile
fi

echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' rsrcs@tiMainString    = '$title                     >> $nclfile
echo 'end if'                                              >> $nclfile
echo ''                                                    >> $nclfile

echo 'dffrnc = scale1*ncvar+addoff1 - (scale2*ncvar2+addoff2)' >> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                           >> $nclfile
fi

if   [ "$prjctntype" = "LL" ]
then
 echo 'dffrnc@lon2d = lon2d'                               >> $nclfile
 echo 'dffrnc@lat2d = lat2d'                               >> $nclfile
 echo 'xplot = gsn_csm_contour_map(xwks,dffrnc,rsrcs)'     >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,dffrnc,rsrcs)'     >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 echo 'xplot = gsn_csm_contour_map(xwks,dffrnc,rsrcs)'     >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,dffrnc,rsrcs)'     >> $nclfile
else
 echo 'xplot = gsn_csm_contour(xwks,dffrnc,rsrcs)'         >> $nclfile
 echo 'eplot = gsn_csm_contour(ewks,dffrnc,rsrcs)'         >> $nclfile
fi
echo ''                                                    >> $nclfile
echo 'end'                                                 >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pd3 -gt 0 ] && [ $pd4 -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps $ncvar1'-'$ncvar2'.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop $ncvar1'-'$ncvar2'.eps'
  $pnmcropbin $ncvar1'-'$ncvar2'.eps001.ppm' >  $ncvar1'-'$ncvar2'.ppm'
  $convertbin $ncvar1'-'$ncvar2'.ppm' -resize $resize $ncvar1'-'$ncvar2'.png'
  rm $ncvar1'-'$ncvar2'.eps001.ppm' $ncvar1'-'$ncvar2'.ppm'
 fi

fi


exit
