
echo ""
echo "           Svector.sh  /  ncl-metno 1.2"
#echo "          Date:        11 April, 2006"
#echo "          Contact:     a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2005-12-12  changed in 3D case, now assumes 3D baroclinic & 2D barotropic
#              (if 4D & baroclinic is stored as one layer on each file,
#               still use 4D!)
# 2006-04-11  implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots;
#             now producing resized cropped png images using pstoppm & convert


if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  Svector.sh <option> <file> <u> <v> <u_bt> <v_bt> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  ...[234]dmap will be displayed on a geogr. map"
 echo ">>>    <file>      name of the netcdf file"
 echo ">>>    <u>         name of variable w/ baroclinic velocity in the x-direction"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>    <v>         name of variable w/ baroclinic velocity in the y-direction"
 echo ">>>    <u_bt>      name of variable w/ barotropic velocity in the x-direction"
 echo ">>>    <v_bt>      name of variable w/ barotropic velocity in the y-direction"
 echo ">>>    <d3node>  node no. of third dimension"
 echo ">>>                if <option> is one of 2d, 2dmap and a fourth"
 echo ">>>                  argument is present, or if <d3node> is negative,"
 echo ">>>                  this will be interpreted as a flag that will cause"
 echo ">>>                  the ncl script to remain"
 echo ">>>    <d4node>  node no. of fourth dimension"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'Svector.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * vector spec.s (size, distance, curly/regular vectors)"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * names of longitude & latitude variables"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Example:"
 echo ">>> ========"
 echo ">>> "
 echo ">>>  Svector.sh 4dmap hydrography.nc u v u_btrop v_btrop 1 10"
 echo ">>>    will produce vectors on a lon-lat grid w/ a map, of"
 echo ">>>     the sums 'u'+'u_btrop' and 'v'+'v_btrop' in the"
 echo ">>>     x- and y-directions, respectively; 'u' and 'v' will be"
 echo ">>>     extracted at the first first node in the third dimension"
 echo ">>>     (usually the top vertical level) and the tenth node in"
 echo ">>>     the fourth dimension (usually time step no. 10),"
 echo ">>>     whereas 'u_btrop' and 'v_btrop' are extracted at the"
 echo ">>>     tenth node in their third dimension; all variables"
 echo ">>>     will be read from the file 'hydrography.nc'"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi



if [ "$6" = "" ]
then

 echo ">>> Syntax error. Correct syntax:"
 echo ">>>  Svector.sh <option> <file> <u> <v> <u_bt> <v_bt> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  ...[234]dmap will be displayed on a geogr. map"
 echo ">>> e.g."
 echo ">>>  Svector.sh 4dmap hydrography.nc u v u_btrop v_btrop 1 10"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   Svector.sh --help"
 echo ">>>  for more details, and an example."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".Svector.ncl"
test -r nclfile && rm nclfile

option=$1
ncfile=$2
ncvaru=$3
ncvarv=$4
ncvarubt=$5
ncvarvbt=$6


if [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$8" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'Svector.sh --help'   for details"
  exit
 fi

 pd3=$7
 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-pd3))
 fi

 pd4=$8
 if [ $pd4 -gt 0 ]
 then
   d4=$pd4
 else
   d4=$((0-pd4))
 fi

 title='":F"+fontno+":'$ncvaru'('$d3','$d4')+'$ncvarubt'('$d3')/'$ncvarv'('$d3','$d4')+'$ncvarvbt'('$d3')"'

 inc3="yes"
 inc4="yes"

elif [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 if [ "$7" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'Svector.sh --help'   for details"
  exit
 fi

 pd3=$7
 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-pd3))
 fi

 title='":F"+fontno+":'$ncvaru'('$d3')+'$ncvarubt'('$d3')/'$ncvarv'('$d3')+'$ncvarvbt'('$d3')"'

 pd4=$pd3
 inc3="yes"
 inc4="no"

elif [ "$option" = "2dmap" ] || [ "$option" = "2d" ]
then

 title='":F"+fontno+":'$ncvaru'+'$ncvarubt'/'$ncvarv'+'$ncvarvbt'"'

 if [ "$7" = "" ]
 then
  pd3="1"
  pd4="1"
 else
  pd3="0"
  pd4="0"
 fi
 inc3="no"
 inc4="no"

else

 echo ">>> Unrecognized <option>, type  'Svector.sh --help'  for details"
 exit

fi

if [ "$option" = "2dmap" ] || [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc4" = "no" ] && [ "$inc3" = "no" ]
then
 dim0string='(:,:)'
 dimstring='(y1:y2,x1:x2)'
 dimbtstring=$dimstring
elif [ "$inc4" = "no" ]
then
 dim0string='(d30,:,:)'
 dimstring='(d30,y1:y2,x1:x2)'
 dimbtstring='(y1:y2,x1:x2)'
else
 dim0string='(d40,d30,:,:)'
 dimstring='(d40,d30,y1:y2,x1:x2)'
 dimbtstring='(d40,y1:y2,x1:x2)'
fi


cat $ncdir/head.ncl                                         > $nclfile

echo 'ncfile = addfile("'$ncfile'","r")'                   >> $nclfile

if [ "$inc3" = "yes" ]
then
 echo 'd30 = '$d3'-1'                                      >> $nclfile
fi
if [ "$inc4" = "yes" ]
then
 echo 'd40 = '$d4'-1'                                      >> $nclfile
fi

echo ''                                                    >> $nclfile
echo 'ncvar0 = ncfile->'$ncvaru$dim0string                 >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                              >> $nclfile
echo 'nx  = nxy(1)'                                        >> $nclfile
echo 'ny  = nxy(0)'                                        >> $nclfile

if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                              >> $nclfile
cat $ncdir/minicolormap.ncl                                >> $nclfile

if   [ "$prjctntype" = "xy" ]
then
 cat $ncdir/limits.ncl                                     >> $nclfile
else
 llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
 ca=`expr index "$llstring" '"'`
 cn=`expr index "${llstring:$ca+1}" '"'`
 lonname=${llstring:ca:cn}
 llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
 ca=`expr index "$llstring" '"'`
 cn=`expr index "${llstring:$ca+1}" '"'`
 latname=${llstring:ca:cn}
 cat $ncdir/lllimits.ncl                                   >> $nclfile
fi

echo ''                                                    >> $nclfile
echo 'ncvar  = ncfile->'$ncvaru$dimstring                  >> $nclfile
echo 'ncvarv = ncfile->'$ncvarv$dimstring                  >> $nclfile
echo 'ncvarubt = ncfile->'$ncvarubt$dimbtstring            >> $nclfile
echo 'ncvarvbt = ncfile->'$ncvarvbt$dimbtstring            >> $nclfile

echo 'if(isatt(ncvar,"_FillValue"))'                       >> $nclfile
echo ' ncvar@_FillValue = ncfile->'$ncvaru'@_FillValue'    >> $nclfile
echo ' ncvarv@_FillValue = ncfile->'$ncvarv'@_FillValue'   >> $nclfile
echo ' ncvarubt@_FillValue = ncfile->'$ncvarubt'@_FillValue'>> $nclfile
echo ' ncvarvbt@_FillValue = ncfile->'$ncvarvbt'@_FillValue'>> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(ncvar,"missing_value"))'                   >> $nclfile
echo '  ncvar@_FillValue = ncfile->'$ncvaru'@missing_value'>> $nclfile
echo '  ncvarv@_FillValue = ncfile->'$ncvarv'@missing_value'>> $nclfile
echo '  ncvarubt@_FillValue = ncfile->'$ncvarubt'@missing_value'>> $nclfile
echo '  ncvarvbt@_FillValue = ncfile->'$ncvarvbt'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"add_offset"))'                       >> $nclfile
echo ' addoffu   = ncfile->'$ncvaru'@add_offset'           >> $nclfile
echo ' addoffv   = ncfile->'$ncvarv'@add_offset'           >> $nclfile
echo ' addoffubt = ncfile->'$ncvarubt'@add_offset'         >> $nclfile
echo ' addoffvbt = ncfile->'$ncvarvbt'@add_offset'         >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoffu   = 0.'                                     >> $nclfile
echo ' addoffv   = 0.'                                     >> $nclfile
echo ' addoffubt = 0.'                                     >> $nclfile
echo ' addoffvbt = 0.'                                     >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"scale_factor"))'                     >> $nclfile
echo ' scaleu   = ncfile->'$ncvaru'@scale_factor'          >> $nclfile
echo ' scalev   = ncfile->'$ncvarv'@scale_factor'          >> $nclfile
echo ' scaleubt = ncfile->'$ncvarubt'@scale_factor'        >> $nclfile
echo ' scalevbt = ncfile->'$ncvarvbt'@scale_factor'        >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scaleu   = 1.'                                      >> $nclfile
echo ' scalev   = 1.'                                      >> $nclfile
echo ' scaleubt = 1.'                                      >> $nclfile
echo ' scalevbt = 1.'                                      >> $nclfile
echo 'end if'                                              >> $nclfile

if [ "$prjctntype" = "ll" ]
then
 cat $ncdir/llsize.ncl                                     >> $nclfile
 cat $ncdir/vector.ncl                                     >> $nclfile
 echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
 echo " lon_min  = ncvar&"$lonname"(0)"                    >> $nclfile
 echo " lon_max  = ncvar&"$lonname"(x2-x1)"                >> $nclfile
 echo " lat_min  = ncvar&"$latname"(0)"                    >> $nclfile
 echo " lat_max  = ncvar&"$latname"(y2-y1)"                >> $nclfile
 echo "else"                                               >> $nclfile
 echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
 echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"                >> $nclfile
 echo " exit"                                              >> $nclfile
 echo "end if"                                             >> $nclfile
 cat $ncdir/llplot.ncl                                     >> $nclfile
 cat $ncdir/vectorpos.ncl                                  >> $nclfile
else
 cat $ncdir/size.ncl                                       >> $nclfile
 cat $ncdir/vector.ncl                                     >> $nclfile
 cat $ncdir/ticklabels.ncl                                 >> $nclfile
fi

echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' rsrcs@tiMainString    = '$title                     >> $nclfile
echo 'end if'                                              >> $nclfile
echo ''                                                    >> $nclfile

echo 'varu=scaleu*ncvar +addoffu + scaleubt*ncvarubt+addoffubt'>> $nclfile
echo 'varv=scalev*ncvarv+addoffv + scalevbt*ncvarvbt+addoffvbt'>> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                           >> $nclfile
fi

if [ "$prjctntype" = "ll" ]
then
 echo 'xplot = gsn_csm_vector_map(xwks,varu,varv,rsrcs)'   >> $nclfile
 echo 'eplot = gsn_csm_vector_map(ewks,varu,varv,rsrcs)'   >> $nclfile
else
 echo 'xplot = gsn_csm_vector(xwks,varu,varv,rsrcs)'       >> $nclfile
 echo 'eplot = gsn_csm_vector(ewks,varu,varv,rsrcs)'       >> $nclfile
fi
echo ''                                                    >> $nclfile
echo 'end'                                                 >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pd3 -gt 0 ] && [ $pd4 -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps 'Suv.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop 'Suv.eps'
  $pnmcropbin 'Suv.eps001.ppm' >  'Suv.ppm'
  $convertbin 'Suv.ppm' -resize $resize 'Suv.png'
  rm 'Suv.eps001.ppm' 'Suv.ppm'
 fi

fi


exit
