
echo ""
echo "       addlayers.sh  /  ncl-metno 1.2"
#echo "          Date:      11 April, 2006"
#echo "          Contact:   a.melsom@met.no"
echo ""

# 2005-05-20  removed map bug by changing from llplot.ncl to llplot1.ncl
# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-11  included option to use 2d fields for lon's & lat's;
#             implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots;
#             now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  addlayers.sh <option> <file> <variable> <firstlevel> <lastlevel> (<d4node>)"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [34]d    - [34]D fields"
 echo ">>>                  [34]dmap - [34]D fields, dims. are lon & lat"
 echo ">>>                  [34]dMap - [34]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[34]d[mM]ap will be displayed on a geogr. map"
 echo ">>>    <file>      name of the netcdf file"
 echo ">>>    <variable>  name of requested variable on the netcdf file"
 echo ">>>                 (case sensitive)"
 echo ">>>    <firstlevel> this is the first vertical level no. in the addition"
 echo ">>>    <lastlevel>  this is the  last vertical level no. in the addition"
 echo ">>>    <d4node>  node no. of fourth dimension (usually time)"
 echo ">>> "
 echo ">>>  Special case:"
 echo ">>>   If <option> is one of [34]dMap, the name of the 2d longitude and"
 echo ">>>    latitude variable may be specified on the command line:"
 echo ">>>   addlayers.sh <option> <lonname> <latname> <file> <variable> (<d4node>)"
 echo ">>>    (Alternatively, if these names are not 'lon' or 'lat', 'userdef.ncl' may"
 echo ">>>     be edited when option is one of [34]dMap.)"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'addlayers.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * names of longitude & latitude variables"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>  (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Example:"
 echo ">>> ========"
 echo ">>> "
 echo ">>>  addlayers.sh 4dmap hydrography.nc thknss 1 5 10"
 echo ">>>    will produce a depiction on a lon-lat grid w/ a map,"
 echo ">>>     of the sum of values for the variable 'thknss' for"
 echo ">>>     layers 1-5 from the tenth time step, provided that"
 echo ">>>     the third and fourth dimensions are layer no. and"
 echo ">>>     time, respectively. Values will be read from the"
 echo ">>>     file 'hydrography.nc'"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$5" = "" ]
then

 echo ">>> Syntax error. Correct syntax:"
 echo ">>>  addlayers.sh <option> <file> <variable> <firstlevel> <lastlevel> (<d4node>)"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [34]d    - [34]D fields"
 echo ">>>                  [34]dmap - [34]D fields, dims. are lon & lat"
 echo ">>>                  [34]dMap - [34]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[34]d[mM]ap will be displayed on a geogr. map"
 echo ">>> e.g."
 echo ">>>  addlayers.sh 4dmap hydrography.nc thknss 1 5 10"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   addlayers.sh --help"
 echo ">>>  for more details, and an example."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

dir=`pwd`
nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".addlayers.ncl"
test -r nclfile && rm nclfile

option=$1
ncfile=$2
if [ "$option" = "4dMap" ] || [ "$option" = "3dMap" ]
then
 dotpos=`expr index "$ncfile" .`
 if [ $dotpos -eq 0 ]
 then
  lonname=$ncfile
  shift
  latname=$2
  shift
  ncfile=$2
 else
  lonname=""
  latname=""
 fi
else
 lonname=""
 latname=""
fi
ncvar=$3


pd3a=$4
if [ $pd3a -gt 0 ]
then
  d3a=$pd3a
else
  d3a=$((0-pd3a))
fi

pd3b=$5
if [ $pd3b -gt 0 ]
then
  d3b=$pd3b
else
  d3b=$((0-pd3b))
fi


if [ "$option" = "4dMap" ] || [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$6" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'addlayers.sh --help'   for details"
  exit
 fi

 pd4=$6
 if [ $pd4 -gt 0 ]
 then
   d4=$pd4
 else
   d4=$((0-pd4))
 fi
 inc4="yes"

 title='":F"+fontno+":'$ncvar', 3rd dim.s no. '$d3a'-'$d3b', 4th dim no. '$d4'"'

elif [ "$option" = "3dMap" ] || [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 title='":F"+fontno+":'$ncvar', 3rd dim. no.s '$d3a'-'$d3b'"'

 pd4=$pd3a
 inc4="no"

else

 echo ">>> Unrecognized <option>, type  'addlayers.sh --help'  for details"
 exit

fi

if   [ "$option" = "3dMap" ] || [ "$option" = "4dMap" ]
then
 prjctntype="LL"
elif [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc4" = "no" ]
then
 dim0string='(d3a,:,:)'
 dimstring='(:,y1:y2,x1:x2)'
else
 dim0string='(d40,d3a,:,:)'
 dimstring='(d40,:,y1:y2,x1:x2)'
fi
dstring='y2-y1+1,x2-x1+1'
xstring='(x1:x2)'
ystring='(y1:y2)'
zstring='(d3-1,:,:)'


cat $ncdir/head.ncl                                       > $nclfile

echo ''                                                  >> $nclfile
echo 'd3a = '$d3a                                        >> $nclfile
echo 'd3b = '$d3b                                        >> $nclfile
echo ''                                                  >> $nclfile

echo 'ncfile = addfile("'$ncfile'","r")'                 >> $nclfile

if [ "$inc4" = "yes" ]
then
 echo 'd40 = '$d4'-1'                                    >> $nclfile
fi

echo ''                                                  >> $nclfile
echo 'ncvar0 = ncfile->'$ncvar$dim0string                >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                            >> $nclfile
echo 'nx  = nxy(1)'                                      >> $nclfile
echo 'ny  = nxy(0)'                                      >> $nclfile

if  [ -e $dir/userdef.ncl ]
then
 userfile=$dir/"userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                            >> $nclfile
cat $ncdir/colormap.ncl                                  >> $nclfile

if   [ "$prjctntype" = "xy" ]
then
 cat $ncdir/limits.ncl                                   >> $nclfile
else
 if [ "$lonname" = "" ]
 then
  llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  lonname=${llstring:ca:cn}
 fi
 if [ "$latname" = "" ]
 then
  llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  latname=${llstring:ca:cn}
 fi
 cat $ncdir/lllimits.ncl                                 >> $nclfile
fi

echo ''                                                  >> $nclfile
echo 'ncvar1 = ncfile->'$ncvar$dimstring                 >> $nclfile

echo 'if(isatt(ncvar1,"_FillValue"))'                    >> $nclfile
echo ' ncvar1@_FillValue = ncfile->'$ncvar'@missing_value'>> $nclfile
echo 'else'                                              >> $nclfile
echo ' if(isatt(ncvar1,"missing_value"))'                >> $nclfile
echo '  ncvar1@_FillValue = ncfile->'$ncvar'@missing_value'>> $nclfile
echo ' end if'                                           >> $nclfile
echo 'end if'                                            >> $nclfile

echo 'if(isatt(ncvar1,"add_offset"))'                    >> $nclfile
echo ' addoff = ncfile->'$ncvar'@add_offset'             >> $nclfile
echo 'else'                                              >> $nclfile
echo ' addoff = 0.'                                      >> $nclfile
echo 'end if'                                            >> $nclfile

echo 'if(isatt(ncvar1,"scale_factor"))'                  >> $nclfile
echo ' scale = ncfile->'$ncvar'@scale_factor'            >> $nclfile
echo 'else'                                              >> $nclfile
echo ' scale = 1.'                                       >> $nclfile
echo 'end if'                                            >> $nclfile

echo 'ncvar = new((/'$dstring'/),float)'                 >> $nclfile

if   [ "$prjctntype" = "LL" ]
then
 cat $ncdir/llsize.ncl                                   >> $nclfile
 cat $ncdir/contour.ncl                                  >> $nclfile
 echo "if (isfilevarcoord(ncfile,\""$ncvar"\",\""$latname"\") .or. isfilevarcoord(ncfile,\""$ncvar"\",\""$lonname"\")) then" >> $nclfile
 echo " print(\"..."$lonname" and/or "$latname" are not 2D fields\")" >> $nclfile
 echo " print(\"   which they must be for the Map options, but\")" >> $nclfile
 echo " print(\"   one or both are (1D) variable coordinate(s)\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"              >> $nclfile
 echo " exit"                                            >> $nclfile
 echo "else"                                             >> $nclfile
 echo " if (isfilevar(ncfile,\""$latname"\") .and. isfilevar(ncfile,\""$lonname"\")) then" >> $nclfile
 echo "  lon2d  = ncfile->"$lonname"(y1:y2,x1:x2)"       >> $nclfile
 echo "  lat2d  = ncfile->"$latname"(y1:y2,x1:x2)"       >> $nclfile
 cat $ncdir/mod_lon.ncl                                  >> $nclfile
 echo " else"                                            >> $nclfile
 echo "  print(\"...could not find a proper set of map variables,\")" >> $nclfile
 echo "  print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo "  system(\"/bin/rm -f ncvar_e.eps\")"             >> $nclfile
 echo "  exit"                                           >> $nclfile
 echo " end if"                                          >> $nclfile
 echo "end if"                                           >> $nclfile
 cat $ncdir/LLplot.ncl                                   >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 cat $ncdir/llsize.ncl                                   >> $nclfile
 cat $ncdir/contour.ncl                                  >> $nclfile
 echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
 echo " lon_min  = ncvar&"$lonname"(0)"                  >> $nclfile
 echo " lon_max  = ncvar&"$lonname"(x2-x1)"              >> $nclfile
 echo " lat_min  = ncvar&"$latname"(0)"                  >> $nclfile
 echo " lat_max  = ncvar&"$latname"(y2-y1)"              >> $nclfile
 echo "else"                                             >> $nclfile
 echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
 echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"              >> $nclfile
 echo " exit"                                            >> $nclfile
 echo "end if"                                           >> $nclfile
 cat $ncdir/llplot.ncl                                   >> $nclfile
else
 cat $ncdir/size.ncl                                     >> $nclfile
 cat $ncdir/contour.ncl                                  >> $nclfile
 cat $ncdir/ticklabels.ncl                               >> $nclfile
fi

echo 'ncvar = 0'                                         >> $nclfile
echo 'do d3 = d3a, d3b'                                  >> $nclfile
echo ' ncvar = ncvar + scale*ncvar1'$zstring'+addoff'    >> $nclfile
echo 'end do'                                            >> $nclfile

echo 'if (mytitle .ne. "auto" ) then'                    >> $nclfile
echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'  >> $nclfile
echo 'else'                                              >> $nclfile
echo ' rsrcs@tiMainString    = '$title                   >> $nclfile
echo 'end if'                                            >> $nclfile
echo ''                                                  >> $nclfile

if  [ -e $dir/usermod.ncl ]
then
 cat $dir/usermod.ncl                                    >> $nclfile
fi

if   [ "$prjctntype" = "LL" ]
then
 echo 'ncvar@lon2d = lon2d'                              >> $nclfile
 echo 'ncvar@lat2d = lat2d'                              >> $nclfile
 echo 'xplot = gsn_csm_contour_map(xwks,ncvar,rsrcs)'    >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,ncvar,rsrcs)'    >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 echo 'xplot = gsn_csm_contour_map(xwks,ncvar,rsrcs)'    >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,ncvar,rsrcs)'    >> $nclfile
else
 echo 'xplot = gsn_csm_contour(xwks,ncvar,rsrcs)'        >> $nclfile
 echo 'eplot = gsn_csm_contour(ewks,ncvar,rsrcs)'        >> $nclfile
fi
echo ''                                                  >> $nclfile
echo 'end'                                               >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pd4 -gt 0 ] && [ $pd3a -gt 0 ] && [ $pd3b -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r $dir/resize ]
 then
  resize=`head -1 $dir/resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps $ncvar'.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop $ncvar'.eps'
  $pnmcropbin $ncvar'.eps001.ppm' >  $ncvar'.ppm'
  $convertbin $ncvar'.ppm' -resize $resize $ncvar'.png'
  rm $ncvar'.eps001.ppm' $ncvar'.ppm'
 fi

fi


exit
