
echo ""
echo "        contour.sh  /  ncl-metno 1.2"
#echo "          Date:     11 April, 2006"
#echo "          Contact:  a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-11  included option to use 2d fields for lon's & lat's;
#             implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots;
#             now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  contour.sh <option> <file> <variable> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  [234]dMap - [234]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[234]d[mM]ap will be displayed on a geogr. map"
 echo ">>>    <file>      name of the netcdf file"
 echo ">>>    <variable>  name of requested variable on the netcdf file"
 echo ">>>                 (case sensitive)"
 echo ">>>    <d3node>  node no. of third dimension"
 echo ">>>                if <option> is one of 2d, 2dmap and a fourth"
 echo ">>>                  argument is present, or if <d3node> is negative,"
 echo ">>>                  this will be interpreted as a flag that will cause"
 echo ">>>                  the ncl script to remain (see examples below)"
 echo ">>>    <d4node>  node no. of fourth dimension"
 echo ">>> "
 echo ">>>  Special case:"
 echo ">>>   If <option> is one of [234]dMap, the name of the 2d longitude and"
 echo ">>>    latitude variable may be specified on the command line:"
 echo ">>>   contour.sh <option> <lonname> <latname> <file> <variable> [<d3node> (<d4node>)]"
 echo ">>>    (Alternatively, if these names are not 'lon' or 'lat', 'userdef.ncl' may"
 echo ">>>     be edited when option is one of [234]dMap.)"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'contour.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * names of longitude & latitude variables"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>  (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Examples:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  contour.sh 4dmap hydrography.nc temp 1 10"
 echo ">>>    will produce a depiction on a lon-lat grid w/ a map,"
 echo ">>>     for the first node in the third dimension (usually"
 echo ">>>     the top vertical level) and the tenth node in the"
 echo ">>>     fourth dimension (usually time step no. 10),"
 echo ">>>     of the variable 'temp' on the file 'hydrography.nc'"
 echo ">>>  contour.sh 3d surface.nc sst -1"
 echo ">>>    will produce a depiction on a x-y grid of the first node"
 echo ">>>     in the third dimension, of the variable 'sst' on the"
 echo ">>>     file 'surface.nc'; and the ncl-script will be retained"
 echo ">>>  contour.sh 2dmap topography.nc Depth a"
 echo ">>>    will produce a depiction on a lon-lat grid w/ a map,"
 echo ">>>    of the variable 'Depth' on the file 'topography.nc';"
 echo ">>>     and the ncl-script will be retained"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$3" = "" ]
then

 echo ">>> Syntax error. Correct syntax:"
 echo ">>>  contour.sh <option> <file> <variable> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  [234]dMap - [234]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[234]d[mM]ap will be displayed on a geogr. map"
 echo ">>> e.g."
 echo ">>>  contour.sh 4dmap hydrography.nc temp 1 10"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   contour.sh --help"
 echo ">>>  for more details, and examples."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".contour.ncl"
test -r nclfile && rm nclfile

option=$1
ncfile=$2
if [ "$option" = "4dMap" ] || [ "$option" = "3dMap" ] || [ "$option" = "2dMap" ]
then
 dotpos=`expr index "$ncfile" .`
 if [ $dotpos -eq 0 ]
 then
  lonname=$ncfile
  shift
  latname=$2
  shift
  ncfile=$2
 else
  lonname=""
  latname=""
 fi
else
 lonname=""
 latname=""
fi
ncvar=$3


if [ "$option" = "4dMap" ] || [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$5" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'contour.sh --help'   for details"
  exit
 fi

 pd3=$4
 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-$pd3))
 fi

 pd4=$5
 if [ $pd4 -gt 0 ]
 then
   d4=$pd4
 else
   d4=$((0-$pd4))
 fi

 title='":F"+fontno+":'$ncvar', 3rd dim. no. '$d3', 4th dim. no. '$d4'"'

 inc3="yes"
 inc4="yes"

elif [ "$option" = "3dMap" ] || [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 if [ "$4" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'contour.sh --help'   for details"
  exit
 fi

 pd3=$4
 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-$pd3))
 fi

 title='":F"+fontno+":'$ncvar', 3rd dim. no. '$d3'"'

 pd4=$pd3
 inc3="yes"
 inc4="no"

elif [ "$option" = "2dMap" ] || [ "$option" = "2dmap" ] || [ "$option" = "2d" ]
then

 title='":F"+fontno+":'$ncvar'"'

 if [ "$4" = "" ]
 then
   pd3="1"
   pd4="1"
 else
   pd3="0"
   pd4="0"
 fi
 inc3="no"
 inc4="no"

else

 echo ">>> Unrecognized <option>, type  'contour.sh --help'  for details"
 exit

fi

if   [ "$option" = "2dMap" ] || [ "$option" = "3dMap" ] || [ "$option" = "4dMap" ]
then
 prjctntype="LL"
elif [ "$option" = "2dmap" ] || [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc4" = "no" ]  && [ "$inc3" = "no" ]
then
 dim0string='(:,:)'
 dimstring='(y1:y2,x1:x2)'
elif [ "$inc4" = "no" ]
then
 dim0string='(d30,:,:)'
 dimstring='(d30,y1:y2,x1:x2)'
else
 dim0string='(d40,d30,:,:)'
 dimstring='(d40,d30,y1:y2,x1:x2)'
fi


cat $ncdir/head.ncl                                       > $nclfile

echo 'ncfile = addfile("'$ncfile'","r")'                 >> $nclfile

if [ "$inc3" = "yes" ]
then
 echo 'd30 = '$d3'-1'                                    >> $nclfile
fi
if [ "$inc4" = "yes" ]
then
 echo 'd40 = '$d4'-1'                                    >> $nclfile
fi

echo ''                                                  >> $nclfile
echo 'ncvar0 = ncfile->'$ncvar$dim0string                >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                            >> $nclfile
echo 'nx  = nxy(1)'                                      >> $nclfile
echo 'ny  = nxy(0)'                                      >> $nclfile

if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                            >> $nclfile
cat $ncdir/colormap.ncl                                  >> $nclfile

if   [ "$prjctntype" = "xy" ]
then
 cat $ncdir/limits.ncl                                   >> $nclfile
else
 if [ "$lonname" = "" ]
 then
  llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  lonname=${llstring:ca:cn}
 fi
 if [ "$latname" = "" ]
 then
  llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  latname=${llstring:ca:cn}
 fi
 cat $ncdir/lllimits.ncl                                 >> $nclfile
fi

echo ''                                                  >> $nclfile
echo 'ncvar  = ncfile->'$ncvar$dimstring                 >> $nclfile

echo 'if(isatt(ncvar,"_FillValue"))'                     >> $nclfile
echo ' ncvar@_FillValue = ncfile->'$ncvar'@_FillValue'   >> $nclfile
echo 'else'                                              >> $nclfile
echo ' if(isatt(ncvar,"missing_value"))'                 >> $nclfile
echo '  ncvar@_FillValue = ncfile->'$ncvar'@missing_value'>> $nclfile
echo ' end if'                                           >> $nclfile
echo 'end if'                                            >> $nclfile

echo 'if(isatt(ncvar,"add_offset"))'                     >> $nclfile
echo ' addoff = ncfile->'$ncvar'@add_offset'             >> $nclfile
echo 'else'                                              >> $nclfile
echo ' addoff = 0.'                                      >> $nclfile
echo 'end if'                                            >> $nclfile

echo 'if(isatt(ncvar,"scale_factor"))'                   >> $nclfile
echo ' scale = ncfile->'$ncvar'@scale_factor'            >> $nclfile
echo 'else'                                              >> $nclfile
echo ' scale = 1.'                                       >> $nclfile
echo 'end if'                                            >> $nclfile


if   [ "$prjctntype" = "LL" ]
then
 cat $ncdir/llsize.ncl                                   >> $nclfile
 cat $ncdir/contour.ncl                                  >> $nclfile
 echo "if (isfilevarcoord(ncfile,\""$ncvar"\",\""$latname"\") .or. isfilevarcoord(ncfile,\""$ncvar"\",\""$lonname"\")) then" >> $nclfile
 echo " print(\"..."$lonname" and/or "$latname" are not 2D fields\")" >> $nclfile
 echo " print(\"   which they must be for the Map options, but\")" >> $nclfile
 echo " print(\"   one or both are (1D) variable coordinate(s)\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"              >> $nclfile
 echo " exit"                                            >> $nclfile
 echo "else"                                             >> $nclfile
 echo " if (isfilevar(ncfile,\""$latname"\") .and. isfilevar(ncfile,\""$lonname"\")) then" >> $nclfile
 echo "  lon2d  = ncfile->"$lonname"(y1:y2,x1:x2)"       >> $nclfile
 echo "  lat2d  = ncfile->"$latname"(y1:y2,x1:x2)"       >> $nclfile
 cat $ncdir/mod_lon.ncl                                  >> $nclfile
 echo " else"                                            >> $nclfile
 echo "  print(\"...could not find a proper set of map variables,\")" >> $nclfile
 echo "  print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo "  system(\"/bin/rm -f ncvar_e.eps\")"             >> $nclfile
 echo "  exit"                                           >> $nclfile
 echo " end if"                                          >> $nclfile
 echo "end if"                                           >> $nclfile
 cat $ncdir/LLplot.ncl                                   >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 cat $ncdir/llsize.ncl                                   >> $nclfile
 cat $ncdir/contour.ncl                                  >> $nclfile
 echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
 echo " lon_min  = ncvar&"$lonname"(0)"                  >> $nclfile
 echo " lon_max  = ncvar&"$lonname"(x2-x1)"              >> $nclfile
 echo " lat_min  = ncvar&"$latname"(0)"                  >> $nclfile
 echo " lat_max  = ncvar&"$latname"(y2-y1)"              >> $nclfile
 echo "else"                                             >> $nclfile
 echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
 echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"              >> $nclfile
 echo " exit"                                            >> $nclfile
 echo "end if"                                           >> $nclfile
 cat $ncdir/llplot.ncl                                   >> $nclfile
else
 cat $ncdir/size.ncl                                     >> $nclfile
 cat $ncdir/contour.ncl                                  >> $nclfile
 cat $ncdir/ticklabels.ncl                               >> $nclfile
fi

echo 'if (mytitle .ne. "auto" ) then'                    >> $nclfile
echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'  >> $nclfile
echo 'else'                                              >> $nclfile
echo ' rsrcs@tiMainString    = '$title                   >> $nclfile
echo 'end if'                                            >> $nclfile
echo ''                                                  >> $nclfile

echo 'var=scale*ncvar+addoff'                            >> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                         >> $nclfile
fi

if   [ "$prjctntype" = "LL" ]
then
 echo 'var@lon2d = lon2d'                                >> $nclfile
 echo 'var@lat2d = lat2d'                                >> $nclfile
 echo 'xplot = gsn_csm_contour_map(xwks,var,rsrcs)'      >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,var,rsrcs)'      >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 echo 'xplot = gsn_csm_contour_map(xwks,var,rsrcs)'      >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,var,rsrcs)'      >> $nclfile
else
 echo 'xplot = gsn_csm_contour(xwks,var,rsrcs)'          >> $nclfile
 echo 'eplot = gsn_csm_contour(ewks,var,rsrcs)'          >> $nclfile
fi
echo ''                                                  >> $nclfile
echo 'end'                                               >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pd3 -gt 0 ] && [ $pd4 -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps $ncvar'.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop $ncvar'.eps'
  $pnmcropbin $ncvar'.eps001.ppm' >  $ncvar'.ppm'
  $convertbin $ncvar'.ppm' -resize $resize $ncvar'.png'
  rm $ncvar'.eps001.ppm' $ncvar'.ppm'
 fi

fi


exit
