
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"   

begin
ncfile = addfile("hycom_expt135.nc","r")

thknss = ncfile->thknss

invar   = ncfile->density
missval = ncfile->density@missing_value
;map_cont       = fspan( 1.9, 2.65,16)
map_cont       = fspan( 1.0, 2.5,16)
;invar   = ncfile->salin
;missval = ncfile->salin@missing_value
;map_cont       = fspan( 33.6, 35.1,16)
;map_cont       = fspan( 32.1, 35.1,16)
lon     = ncfile->lon
lat     = ncfile->lat
dims  = dimsizes(thknss)
nx    = dims(2)
nz    = 50
btmno = dims(1)
altno = 70
tno   = 97

nx = nx/2

zum   = new((/btmno,nx/),float)
ncvar = new((/nz,nx/),float)
ncvar@_FillValue = missval
level = new((/nz/),float)
layer = new((/nz,nx/),integer)

zum(0,0:nx-1) = thknss(tno-1,0,0:nx-1,altno-1)
do z = 1, btmno-1
  zum(z,0:nx-1) = zum(z-1,:) + thknss(tno-1,z,0:nx-1,altno-1)
end do

bottom = max(zum)
dz = bottom/nz
level(0) = bottom - dz/2
do z = 1, nz-1
 level(z) = level(z-1) - dz
end do

first = 0
first = 3*nx/5

do while (ismissing(thknss(tno-1,0,first,altno-1)))
   first = first + 1
end do

last = nx-1
do while (ismissing(thknss(tno-1,0,last,altno-1)))
   last = last - 1
end do

do x = first, last
  no = btmno
  if( ismissing(thknss(tno-1,0,x,altno-1))) then
    do z = 0, nz-1
      layer(z,x) = btmno
    end do
  else
    do z = 0, nz-1
      if( no .gt. 1) then
        do while( zum(no-1,x) .gt. level(z) .and. no .gt. 1)
          no = no - 1
        end do
      end if
      if( no .eq. 1 .and. zum(no-1,x) .gt. level(z)) then
        no = no - 1
      end if
      layer(z,x) = no
    end do
  end if
end do

do x = first, last
  do z = 0, nz-1
    thislayer = layer(z,x)
    zz = nz - z - 1
    if( thislayer .eq. btmno) then
      ncvar(z,x) = ncvar@_FillValue
    else
      ncvar(z,x) = invar(tno-1,thislayer,x,altno-1)
    end if
  end do
end do

title=":F22:" + invar@long_name
title= title  + ", time no. "
title= title  + sprinti("%4.4i", tno)
title= title  + ", lon. "
title= title  + sprintf("%6.2f", lon(altno))

xwks = gsn_open_wks("x11","ncvar_x") ; Open workstations
ewks = gsn_open_wks("eps","ncvar_e")


gsn_define_colormap(xwks, "BkBlAqGrYeOrReViWh200") ; Get preliminary color map
cmaporg      = gsn_retrieve_colormap(xwks)         ; Store preset colormap

cmap         = new((/22,3/),float)
cmap(0,:)    = (/1. ,1. ,1./)        ; Set background to white
cmap(1,:)    = (/0. ,0. ,0./)        ;  and foreground to black
cmap(2:20,:) = cmaporg(24:168:8,:)   ; Set active colors

gsn_define_colormap(xwks, cmap)      ; Set color map
gsn_define_colormap(ewks, cmap)


rsrcs                 = True                  ; Set resources

rsrcs@cnInfoLabelOn   = False
rsrcs@cnLineLabelsOn  = False
rsrcs@cnFillOn        = True

rsrcs@pmLabelBarDisplayMode = "Always"        ; Turn on label bar


rsrcs@lbAutoManage    = False
rsrcs@lbPerimOn       = False
rsrcs@lbLabelFontHeightF    = 0.02
rsrcs@lbLabelFont     = 22

;  Set subplot def.s:
 height = 0.6
 width  = 0.6
 xp     = 0.15
 yp     = 0.9

  rsrcs@vpHeightF  = height  ; Define height, width, and location of plot
  rsrcs@vpWidthF   = width
  rsrcs@vpXF       = xp
  rsrcs@vpYF       = yp

rsrcs@sfXArray = lat(first:last)
rsrcs@sfYArray = -level
rsrcs@cnLevelSelectionMode = "ExplicitLevels"
rsrcs@cnLevels = map_cont

rsrcs@tiMainString    = title


xplot = gsn_contour(xwks,ncvar(:,first:last),rsrcs)
eplot = gsn_contour(ewks,ncvar(:,first:last),rsrcs)


                                          ; CLEAN UP:
delete(xplot)
delete(eplot)
delete(ncvar)
delete(cmaporg)
delete(cmap)
delete(rsrcs)


end
