
echo ""
echo "        layersection.sh  /  ncl-metno 1.2"
#echo "          Date:          7 April, 2006"
#echo "          Contact:       a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-12  corrected dubious interpretation of negative input node no
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2005-11-21  exported part of the script to override.ncl & layers_on.ncl
# 2006-04-07  now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> NOTE: This script assumes that the variable to depict has been"
 echo ">>>  stored with dimensions in the order x  -y  -layer(-time)"
 echo ">>>                                   or lon-lat-layer(-time)"
 echo ">>>  (w/ Fortran style sequence of dimensions)."
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  layersection.sh (layer) <hfile> <varfile> <hname> <varname>\\"
 echo ">>>                                            <dimname> <node> <time>" 
 echo ">>> where"
 echo ">>>    (layer)     include an initial argument 'layer' if you want isopleths"
 echo ">>>                 to be drawn for layer interfaces, ignore otherwise"
 echo ">>>    <hfile>     name of netCDF file w/thickness results"
 echo ">>>    <varfile>   name of netCDF file w/requested variable"
 echo ">>>                 you may use '.' if both variables are on the same file"
 echo ">>>    <hname>     name of thickness variable"
 echo ">>>    <varname>   name of variable to depict"
 echo ">>>             NOTE! This script requires the existence of an"
 echo ">>>                   attribute 'missing_value' to <varname>"
 echo ">>>    <dimname>   name of crossection's horizontal dimension"
 echo ">>>                 e.g.,  lat  for a lat-z (meridional) crossection"
 echo ">>>    <node>      node of non-depicted dimension"
 echo ">>>                 if <dimname> is  lat , this is the  lon  grid no."
 echo ">>>    <time>      time step no. to depict (use  0  for x-y-z & lon-lat-z fields)"
 echo ">>>                a negative <time> value is interpreted as a flag that"
 echo ">>>                 stops the ncl script from being deleted"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'layersection.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Examples:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  layersection.sh layer hycom_expt007.nc . thknss salin lat 30 49"
 echo ">>>    will produce a lat-z crossection normal to longitude node no. 30,"
 echo ">>>     of the variable 'salin' on the file 'hycom_expt007.nc',"
 echo ">>>     w/ layers given by 'thknss' on the same file, add isopleths for"
 echo ">>>     layer interfaces, from timestep no. 49"
 echo ">>>  layersection.sh hycom_expt007.nc . thknss . lat 30 49"
 echo ">>>    same as above, but here, thknss is requested; this is a"
 echo ">>>     special case where the layer no. will be contoured"
 echo ">>>     (the thickess will correspond to the distance between layer"
 echo ">>>      interfaces), isopleths for layer interfaces will not be drawn"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi


# Should contour lines for layers be added?
arg=$1
if [ "$arg" = "layer" ] || [ "$arg" = "Layer" ] || [ "$arg" = "LAYER" ]
then
 layers_on="1"
 shift
 arg=$1
else
 layers_on="0"
fi



if [ "$7" = "" ]
then

 echo ">>> "
 echo ">>> NOTE: This script assumes that the variable to depict has been"
 echo ">>>  stored with dimensions in the order x  -y  -layer(-time)"
 echo ">>>                                   or lon-lat-layer(-time)"
 echo ">>>  (w/ Fortran style sequence of dimensions)."
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  layersection.sh (layer) <hfile> <varfile> <hname> <varname>\\"
 echo ">>>                                            <dimname> <node> <time>" 
 echo ">>> e.g. (for a meridional crossection)"
 echo ">>>  layersection.sh hycom_expt007.nc . thknss salin lat 30 49"
 echo ">>> "
 echo ">>> Note: specify  0  for the <time> argument for x-y-z & lon-lat-z fields"
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   layersection.sh --help"
 echo ">>>  for more details, and examples."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".layersection.ncl"
test -r nclfile && rm nclfile


hfile=$arg
vfile=$2
hname=$3
vname=$4
dname=$5
pnode=$6
ptime=$7



if [ "$2" = "." ]
then
  vfile=$hfile
fi

if [ "$vname" = "." ]
then
  vname=$hname
fi

# reset to act as flag?
# ...node no.:
if [ $pnode -gt 0 ]
then
  node=$pnode
else
  node=$((0-pnode))
fi
# ...time step no.:
if [ "$ptime" = "-0" ]
then
  ptime="-1"
  time="0"
elif [ $ptime -gt 0 ]
then
  time=$ptime
else
  time=$((0-ptime))
fi


if [ "$dname" = "lon" ]
then
  altname="lat"
elif [ "$dname" = "lat" ]
then
  altname="lon"
fi


if [ $time -eq 0 ]
then
  if [ "$dname" = "x" ] || [ "$dname" = "lon" ]
  then
    dim0string='(:,'$node',:)'
    dimstring='(:,'$node',x1:x2)'
  else
    dim0string='(:,:,'$node')'
    dimstring='(:,x1:x2,'$node')'
  fi
else
  if [ "$dname" = "x" ] || [ "$dname" = "lon" ]
  then
    dim0string='('$time'-1,:,'$node',:)'
    dimstring='('$time'-1,:,'$node',x1:x2)'
  else
    dim0string='('$time'-1,:,:,'$node')'
    dimstring='('$time'-1,:,x1:x2,'$node')'
  fi
fi


cat $ncdir/head.ncl                                         > $nclfile

echo 'ncfile = addfile("'$hfile'","r")'                    >> $nclfile

echo 'ncvar0 = ncfile->'$hname$dim0string                  >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                              >> $nclfile
echo 'nx  = nxy(1)'                                        >> $nclfile
echo 'ny  = nxy(0)'                                        >> $nclfile


if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                              >> $nclfile
cat $ncdir/colormap.ncl                                    >> $nclfile
cat $ncdir/limits.ncl                                      >> $nclfile

echo 'ncfile2 = addfile("'$vfile'","r")'                   >> $nclfile
echo ''                                                    >> $nclfile
echo 'thknss = ncfile->'$hname$dimstring                   >> $nclfile
echo 'invar  = ncfile2->'$vname$dimstring                  >> $nclfile


if [ "$dname" = "lon" ]
then
  echo 'lon = ncfile->lon(x1:x2)'                          >> $nclfile
  echo 'lat = ncfile->lat(:)'                              >> $nclfile
elif [ "$dname" = "lat" ]
then
  echo 'lon = ncfile->lon(:)'                              >> $nclfile
  echo 'lat = ncfile->lat(x1:x2)'                          >> $nclfile
fi

echo 'if(isatt(thknss,"_FillValue"))'                      >> $nclfile
echo ' thknss@_FillValue = ncfile->'$hname'@_FillValue'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(thknss,"missing_value"))'                  >> $nclfile
echo '  thknss@_FillValue = ncfile->'$hname'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(invar,"_FillValue"))'                       >> $nclfile
echo ' invar@_FillValue = ncfile2->'$vname'@_FillValue'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(thknss,"missing_value"))'                  >> $nclfile
echo '  invar@_FillValue = ncfile2->'$vname'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(thknss,"add_offset"))'                      >> $nclfile
echo ' addoffh = ncfile->'$hname'@add_offset'              >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoffh = 0.'                                       >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(invar,"add_offset"))'                       >> $nclfile
echo ' addoffv = ncfile2->'$vname'@add_offset'             >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoffv = 0.'                                       >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(thknss,"scale_factor"))'                    >> $nclfile
echo ' scaleh = ncfile->'$hname'@scale_factor'             >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scaleh = 1.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(invar,"scale_factor"))'                     >> $nclfile
echo ' scalev = ncfile2->'$vname'@scale_factor'            >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scalev = 1.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'xp = xp + 0.01'                                      >> $nclfile
echo 'width  = maxsize'                                    >> $nclfile
echo 'height = maxsize'                                    >> $nclfile
cat $ncdir/contour.ncl                                     >> $nclfile

echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
echo ' title=":F"+fontno+":" + mytitle'                    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' title=":F"+fontno+":" + invar@long_name'            >> $nclfile
if [ "$time" != "0" ]
then
 echo ' title= title  + ", time no. "'                     >> $nclfile
 echo ' title= title  + sprinti("%4.4i", '$time')'         >> $nclfile
fi
if [ "$altname" = "lon" ] || [ "$altname" = "lat" ]
then
 echo ' node = '$node                                      >> $nclfile
 echo ' title= title  + ", '$altname'. "'                  >> $nclfile
 echo ' title= title  + sprintf("%6.2f", '$altname'(node))'>> $nclfile
fi
echo 'end if'                                              >> $nclfile
echo ''                                                    >> $nclfile


cat $ncdir/layersection.ncl                                >> $nclfile

if [ "$dname" = "lon" ]
then
  echo 'if( lon(last) .gt. 360.) then'                     >> $nclfile
  echo '  rsrcs@sfXArray = lon(first:last) - 360.'         >> $nclfile
  echo 'else'                                              >> $nclfile
  echo '  rsrcs@sfXArray = lon(first:last)'                >> $nclfile
  echo 'end if'                                            >> $nclfile
elif [ "$dname" = "lat" ]
then
  echo 'rsrcs@sfXArray = lat(first:last)'                  >> $nclfile
fi

echo 'rsrcs@sfYArray = -level'                             >> $nclfile
cat $ncdir/ticklabels.ncl                                  >> $nclfile

echo '; ***override  mapcont  & reset  ncvar when layer no. is requested***'>> $nclfile
if [ "$hname" = "$vname" ]
then
  cat $ncdir/override.ncl                                  >> $nclfile
fi
if [ $layers_on -eq 1 ]
then
  cat $ncdir/layers_on.ncl                                 >> $nclfile
fi

echo 'rsrcs@tiMainString    = title'                       >> $nclfile
echo ''                                                    >> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                           >> $nclfile
fi

if [ $layers_on -eq 1 ]
then
 echo 'rsrcs@gsnFrame        = False'                      >> $nclfile
fi

echo 'xplot = gsn_csm_contour(xwks,ncvar(:,first:last),rsrcs)' >> $nclfile
echo 'eplot = gsn_csm_contour(ewks,ncvar(:,first:last),rsrcs)' >> $nclfile
echo ''                                                    >> $nclfile

if [ $layers_on -eq 1 ]
then
 cat $ncdir/layerlines.ncl                                   >> $nclfile
fi

echo 'end'                                                 >> $nclfile


$nclbin < $nclfile

echo ''


if [ $ptime -ge 0 ] && [ $pnode -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps $vname'.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop $vname'.eps'
  $pnmcropbin $vname'.eps001.ppm' >  $vname'.ppm'
  $convertbin $vname'.ppm' -resize $resize $vname'.png'
  rm $vname'.eps001.ppm' $vname'.ppm'
 fi

fi


exit
