
echo ""
echo "        mcontour.sh  /  ncl-metno 1.2"
#echo "          Date:      11 April, 2006"
#echo "          Contact:   a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-11  included option to use 2d fields for lon's & lat's;
#             implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots
#             now producing a gif animation, using gifsicle & more


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> NOTE: The user is **STRONGLY** recommended to copy"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'mcontour.sh' is given,"
 echo ">>>  and at least specify"
 echo ">>>   * color map (palette)"
 echo ">>>  -otherwise, the series of files produced by  mcontour.sh"
 echo ">>>   will (usually) NOT have the same color map (see more information below)"
 echo ">>> "
 echo ">>> NOTE: This script will provide a set of output files, where"
 echo ">>>  the value of the final (3rd or 4th) dimension changes from"
 echo ">>>  one output file to the next. Below, we assume that this"
 echo ">>>  dimension is time."
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>> ...if  <option> is  4d ,  4dmap  or  4dMap:"
 echo ">>>  mcontour.sh <option> <file> <variable> <depth> <first> <last> (<step>) (-<delay>)"
 echo ">>> ...if  <option> is  3d ,  3dmap  or  3dMap :"
 echo ">>>  mcontour.sh <option> <file> <variable> <first> <last> (<step>) (-<delay>)"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [34]d    - [34]D fields"
 echo ">>>                  [34]dmap - [34]D fields, dims. are lon & lat"
 echo ">>>                  [34]dMap - [34]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[34]d[mM]ap will be displayed on a geogr. map"
 echo ">>>    <file>      name of the netcdf file"
 echo ">>>    <variable>  name of requested variable on the netcdf file"
 echo ">>>                 (case sensitive)"
 echo ">>>    <depth>     vertical level no."
 echo ">>>    <first>     first time step no."
 echo ">>>    <last>      last  time step no."
 echo ">>>    <step>      time step between consequtive frames"
 echo ">>>                 (optional, set to  1  if not specified by user)"
 echo ">>>    <delay>     frame delay in ms (optional, set to  10  if not specified by user)"
 echo ">>> "
 echo ">>>  Special case:"
 echo ">>>   If <option> is one of [34]dMap, the name of the 2d longitude and"
 echo ">>>    latitude variable may be specified on the command line, e.g. (3D fields):"
 echo ">>>   mcontour.sh <option> <lonname> <latname> <file> <variable> <first> <last> (<step>) (-<delay>)"
 echo ">>>    (Alternatively, if these names are not 'lon' or 'lat', 'userdef.ncl' may"
 echo ">>>     be edited when option is one of [34]dMap.)"
 echo ">>> "
 echo ">>>  The script will produce a gif animation."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'mcontour.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * names of longitude & latitude variables"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Examples:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  mcontour.sh 4dmap hydrography.nc temp 1 10 15"
 echo ">>>    will produce a gif animation with 6 frames for time steps"
 echo ">>>      10-15 of the first vertical level of the variable 'temp'"
 echo ">>>     on the file 'hydrography.nc', on a lon-lat grid w/ a map"
 echo ">>>  mcontour.sh 3d surface.nc sst 1 9 2"
 echo ">>>    will produce a gif animation with 5 frames for time steps"
 echo ">>>    1, 3, 5, 7 and 9 on a x-y grid of the variable 'sst' on"
 echo ">>>    the file 'surface.nc'"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$5" = "" ]
then

 echo ">>> NOTE: The user is **STRONGLY** recommended to copy"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'mcontour.sh' is given,"
 echo ">>>  and at least specify"
 echo ">>>   * color map (palette)"
 echo ">>>  -otherwise, the series of files produced by  mcontour.sh"
 echo ">>>   will (usually) NOT have the same color map, details are provided by executing"
 echo ">>>   mcontour.sh --help"
 echo ">>> "
 echo ">>> "
 echo ">>> Syntax error. Correct syntax:"
 echo ">>> ...if  <option> is  4d  or  4dmap:"
 echo ">>>  mcontour.sh <option> <file> <variable> <depth> <first> <last> (<step>)"
 echo ">>> ...if  <option> is   3d  or   3dmap:"
 echo ">>>  mcontour.sh <option> <file> <variable> <first> <last> (<step>)"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [34]d    - [34]D fields"
 echo ">>>                  [34]dmap - [34]D fields, dims. are lon & lat"
 echo ">>>                  [34]dMap - [34]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[34]d[mM]ap will be displayed on a geogr. map"
 echo ">>> e.g."
 echo ">>>  mcontour.sh 4dmap hydrography.nc temp 1 10 15"
 echo ">>> "
 echo ">>>  The script will produce a gif animation."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   mcontour.sh --help"
 echo ">>>  for more details, and examples."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
p2gbin=`which ppmtogif`
animbin=`which gifsicle`
giffiles=""

nclfile=".mcontour.ncl"
test -r nclfile && rm nclfile

option=$1
ncfile=$2
if [ "$option" = "4dMap" ] || [ "$option" = "3dMap" ] || [ "$option" = "2dMap" ]
then
 dotpos=`expr index "$ncfile" .`
 if [ $dotpos -eq 0 ]
 then
  lonname=$ncfile
  shift
  latname=$2
  shift
  ncfile=$2
 else
  lonname=""
  latname=""
 fi
else
 lonname=""
 latname=""
fi
ncvar=$3


if [ "$option" = "4dMap" ] || [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$6" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'mcontour.sh --help'   for details"
  exit
 fi

 level=$4
 firsttime=$5
 lasttime=$6
 value=$7
 if [ "$value" = "" ]
 then
  step="1"
  delay="10"
 else
  if [ $value -le 0 ]
  then
   step="1"
   delay=$((-value))
  else
   step=$value
   value=$8
  fi
  if [ "$value" = "" ]
  then
   delay="10"
  else
   delay=$((-value))
  fi
 fi

 inc_z="yes"

elif [ "$option" = "3dMap" ] || [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 firsttime=$4
 lasttime=$5

 value=$6
 if [ "$value" = "" ]
 then
  step="1"
  delay="10"
 else
  if [ $value -le 0 ]
  then
   step="1"
   delay=$((-value))
  else
   step=$value
   value=$7
  fi
  if [ "$value" = "" ]
  then
   delay="10"
  else
   delay=$((-value))
  fi
 fi

 inc_z="no"

else

 echo ">>> Unrecognized <option>, type  'mcontour.sh --help'  for details"
 exit

fi


# ...set flags:
if [ $firsttime -lt 0 ]
then
 firsttime=$((-firsttime))
 keep_gif="1"
else
 keep_gif="0"
fi
if [ $lasttime -lt 0 ]
then
 lasttime=$((-lasttime))
 keep_ppm="1"
else
 keep_ppm="0"
fi

# ...check sequence:
if [ $firsttime -gt $lasttime ]
then
 echo "Illegal syntax for time stepping. Terminates."
 exit
fi


if   [ "$option" = "3dMap" ] || [ "$option" = "4dMap" ]
then
 prjctntype="LL"
elif [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc_z" = "no" ]
then
 dim0string='(t0,:,:)'
 dimstring='(t0,y1:y2,x1:x2)'
else
 dim0string='(t0,z0,:,:)'
 dimstring='(t0,z0,y1:y2,x1:x2)'
fi



###  MAIN TIME LOOP STARTS BELOW


time=$firsttime

while true
do


 if [ $time -gt 999 ]
 then
  ttitle='":F"+fontno+":'$ncvar', timestep no. '$time
 elif [ $time -gt 99 ]
 then
  ttitle='":F"+fontno+":'$ncvar', timestep no. 0'$time
 elif [ $time -gt 9 ]
 then
  ttitle='":F"+fontno+":'$ncvar', timestep no. 00'$time
 else
  ttitle='":F"+fontno+":'$ncvar', timestep no. 000'$time
 fi
 if [ "$option" = "4dMap" ] || [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
 then
  title=$ttitle', level no. '$level'"'
 else
  title=$ttitle'"'
 fi
 

 cat $ncdir/mhead.ncl                                      > $nclfile

 echo 'ncfile = addfile("'$ncfile'","r")'                 >> $nclfile

 echo 't0 = '$time'-1'                                    >> $nclfile
 if [ "$inc_z" = "yes" ]
 then
  echo 'z0 = '$level'-1'                                  >> $nclfile
 fi

 echo ''                                                  >> $nclfile
 echo 'ncvar0 = ncfile->'$ncvar$dim0string                >> $nclfile
 echo 'nxy = dimsizes(ncvar0)'                            >> $nclfile
 echo 'nx  = nxy(1)'                                      >> $nclfile
 echo 'ny  = nxy(0)'                                      >> $nclfile

 if  [ -e userdef.ncl ]
 then
  userfile="userdef.ncl"
 else
  userfile=$ncdir/"userdef.ncl"
 fi
 cat $userfile                                            >> $nclfile
 cat $ncdir/colormap.ncl                                  >> $nclfile

 if   [ "$prjctntype" = "xy" ]
 then
  cat $ncdir/limits.ncl                                   >> $nclfile
 else
  if [ "$lonname" = "" ]
  then
   llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
   ca=`expr index "$llstring" '"'`
   cn=`expr index "${llstring:$ca+1}" '"'`
   lonname=${llstring:ca:cn}
  fi
  if [ "$latname" = "" ]
  then
   llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
   ca=`expr index "$llstring" '"'`
   cn=`expr index "${llstring:$ca+1}" '"'`
   latname=${llstring:ca:cn}
  fi
  cat $ncdir/lllimits.ncl                                 >> $nclfile
 fi

 echo ''                                                  >> $nclfile
 echo 'ncvar  = ncfile->'$ncvar$dimstring                 >> $nclfile

 echo 'if(isatt(ncvar,"_FillValue"))'                     >> $nclfile
 echo ' ncvar@_FillValue = ncfile->'$ncvar'@_FillValue'   >> $nclfile
 echo 'else'                                              >> $nclfile
 echo ' if(isatt(ncvar,"missing_value"))'                 >> $nclfile
 echo '  ncvar@_FillValue = ncfile->'$ncvar'@missing_value'>> $nclfile
 echo ' end if'                                           >> $nclfile
 echo 'end if'                                            >> $nclfile

 echo 'if(isatt(ncvar,"add_offset"))'                     >> $nclfile
 echo ' addoff = ncfile->'$ncvar'@add_offset'             >> $nclfile
 echo 'else'                                              >> $nclfile
 echo ' addoff = 0.'                                      >> $nclfile
 echo 'end if'                                            >> $nclfile

 echo 'if(isatt(ncvar,"scale_factor"))'                   >> $nclfile
 echo ' scale = ncfile->'$ncvar'@scale_factor'            >> $nclfile
 echo 'else'                                              >> $nclfile
 echo ' scale = 1.'                                       >> $nclfile
 echo 'end if'                                            >> $nclfile

 if   [ "$prjctntype" = "LL" ]
 then
  cat $ncdir/llsize.ncl                                   >> $nclfile
  cat $ncdir/mcontour.ncl                                 >> $nclfile
  echo "if (isfilevarcoord(ncfile,\""$ncvar"\",\""$latname"\") .or. isfilevarcoord(ncfile,\""$ncvar"\",\""$lonname"\")) then" >> $nclfile
  echo " print(\"..."$lonname" and/or "$latname" are not 2D fields\")" >> $nclfile
  echo " print(\"   which they must be for the Map options, but\")" >> $nclfile
  echo " print(\"   one or both are (1D) variable coordinate(s)\")" >> $nclfile
  echo " system(\"/bin/rm -f ncvar_e.eps\")"              >> $nclfile
  echo " exit"                                            >> $nclfile
  echo "else"                                             >> $nclfile
  echo " if (isfilevar(ncfile,\""$latname"\") .and. isfilevar(ncfile,\""$lonname"\")) then" >> $nclfile
  echo "  lon2d  = ncfile->"$lonname"(y1:y2,x1:x2)"       >> $nclfile
  echo "  lat2d  = ncfile->"$latname"(y1:y2,x1:x2)"       >> $nclfile
  cat $ncdir/mod_lon.ncl                                  >> $nclfile
  echo " else"                                            >> $nclfile
  echo "  print(\"...could not find a proper set of map variables,\")" >> $nclfile
  echo "  print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
  echo "  system(\"/bin/rm -f ncvar_e.eps\")"             >> $nclfile
  echo "  exit"                                           >> $nclfile
  echo " end if"                                          >> $nclfile
  echo "end if"                                           >> $nclfile
  cat $ncdir/LLplot.ncl                                   >> $nclfile
 elif [ "$prjctntype" = "ll" ]
 then
  cat $ncdir/llsize.ncl                                   >> $nclfile
  cat $ncdir/mcontour.ncl                                 >> $nclfile
  echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
  echo " lon_min  = ncvar&"$lonname"(0)"                  >> $nclfile
  echo " lon_max  = ncvar&"$lonname"(x2-x1)"              >> $nclfile
  echo " lat_min  = ncvar&"$latname"(0)"                  >> $nclfile
  echo " lat_max  = ncvar&"$latname"(y2-y1)"              >> $nclfile
  echo "else"                                             >> $nclfile
  echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
  echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
  echo " system(\"/bin/rm -f ncvar_e.eps\")"              >> $nclfile
  echo " exit"                                            >> $nclfile
  echo "end if"                                           >> $nclfile
  cat $ncdir/llplot.ncl                                   >> $nclfile
 else
  cat $ncdir/size.ncl                                     >> $nclfile
  cat $ncdir/mcontour.ncl                                 >> $nclfile
  cat $ncdir/ticklabels.ncl                               >> $nclfile
 fi

 echo 'if (mytitle .ne. "auto" ) then'                    >> $nclfile
 echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'  >> $nclfile
 echo 'else'                                              >> $nclfile
 echo ' rsrcs@tiMainString    = '$title                   >> $nclfile
 echo 'end if'                                            >> $nclfile
 echo ''                                                  >> $nclfile

 echo 'rsrcs@gsnFrame        = False'                     >> $nclfile
 echo ''                                                  >> $nclfile
 echo 'var=scale*ncvar+addoff'                            >> $nclfile


 if  [ -e usermod.ncl ]
 then
  cat usermod.ncl                                         >> $nclfile
 fi

 if   [ "$prjctntype" = "LL" ]
 then
  echo 'var@lon2d = lon2d'                                >> $nclfile
  echo 'var@lat2d = lat2d'                                >> $nclfile
  echo 'eplot = gsn_csm_contour_map(ewks,var,rsrcs)'      >> $nclfile
 elif [ "$prjctntype" = "ll" ]
 then
  echo 'eplot = gsn_csm_contour_map(ewks,var,rsrcs)'      >> $nclfile
 else
  echo 'eplot = gsn_csm_contour(ewks,var,rsrcs)'          >> $nclfile
 fi
 echo ''                                                  >> $nclfile
 echo 'frame(ewks)'                                       >> $nclfile


 cat $ncdir/del_mcontour.ncl                              >> $nclfile


 $nclbin < $nclfile

 echo ""
 echo ""

 rm $nclfile

 if [ $time -gt 999 ]
 then
  ttext=$time
 elif [ $time -gt 99 ]
 then
  ttext="0"$time
 elif [ $time -gt 9 ]
 then
  ttext="00"$time
 else
  ttext="000"$time
 fi

 if [ -r ncvar_e.eps ]
 then

  if [ -r resize ]
  then
   resize=`head -1 resize`
  else
   resize=`head -1 $ncdir/resize`
  fi

  if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$p2gbin" == "" ] || [ "$animbin" == "" ]
  then
   echo "...missing one or more of the binaries"
   echo "    pstopnm  pnmcrop  ppm2gif  gifsicle"
   echo "...terminating w/ no action taken"
   exit
  else
   $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop 'ncvar_e.eps'
   $pnmcropbin 'ncvar_e.eps001.ppm' >  $ncvar$ttext'.ppm'
   $p2gbin $ncvar$ttext'.ppm' > $ncvar$ttext'.gif'
   rm 'ncvar_e.eps' 'ncvar_e.eps001.ppm'
   if [ $keep_ppm -eq 0 ]
   then
    rm $ncvar$ttext'.ppm'
   fi
   giffiles=$giffiles" "$ncvar$ttext'.gif'
  fi

 fi

 time=$((time+step))
 if [ $time -gt $lasttime ]
 then
  break
 fi

done


# ...make 4 additional copies of last frame to pause animation briefly
finaltime=$((time+step+step+step))
otext=$ttext
while [ $time -le $finaltime ]
do
 if [ $time -gt 999 ]
 then
  ttext=$time
 elif [ $time -gt 99 ]
 then
  ttext="0"$time
 elif [ $time -gt 9 ]
 then
  ttext="00"$time
 else
  ttext="000"$time
 fi
 cp $ncvar$otext'.gif' $ncvar$ttext'.gif'
 giffiles=$giffiles" "$ncvar$ttext'.gif'
 time=$((time+step))
done

# ...create gif animation
$animbin --loop -O2 -d $delay $giffiles > $ncvar.igif

if [ $keep_gif -eq 0 ]
then
 /bin/rm $giffiles
fi


exit
