
echo ""
echo "        mlayersection.sh  /  ncl-metno 1.2"
#echo "          Date:           13 April 2005"
#echo "          Contact:        a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-03-11  implemented ability to produce multiple images as a
#              function of the non-displayed horizontal coordinate
# 2006-04-13  modified use of longitude and latitude coordinates
#             now producing a gif animation, using gifsicle & more




# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> NOTE: The user is **STRONGLY** recommended to copy"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'mlayersection.sh' is given,"
 echo ">>>  and at least specify"
 echo ">>>   * color map (palette)"
 echo ">>>  -otherwise, the series of files produced by  mlayersection.sh"
 echo ">>>   will (usually) NOT have the same color map (see more information below)"
 echo ">>> "
 echo ">>> NOTE: This script assumes that the variable to depict has been"
 echo ">>>  stored with dimensions in the order x  -y  -layer(-time)"
 echo ">>>                                   or lon-lat-layer(-time)"
 echo ">>>  (w/ Fortran style sequence of dimensions)."
 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  mlayersection.sh <hfile> <varfile> <hname> <varname> <dimname> <node> <first> <last> (<step>) (-<delay>)"
 echo ">>> where"
 echo ">>>    <hfile>     name of netCDF file w/thickness results"
 echo ">>>    <varfile>   name of netCDF file w/requested variable"
 echo ">>>                 you may use '.' if both variables are on the same file"
 echo ">>>    <hname>     name of thickness variable"
 echo ">>>    <varname>   name of variable to depict"
 echo ">>>             NOTE! This script requires the existence of an"
 echo ">>>                   attribute 'missing_value' to <varname>"
 echo ">>>    <dimname>   name of crossection's horizontal dimension"
 echo ">>>                 NOTE: If using longitude or latitude for <dimname>,"
 echo ">>>                        specify 'lon' or 'lat', the actual name will"
 echo ">>>                        be read from the file 'userdef.ncl'"
 echo ">>>    <node>      node of non-depicted dimension"
 echo ">>>                 if <dimname> is  lat , this is the  lon  grid no."
 echo ">>>                NOTE: use  0  for x-y-z & lon-lat-z fields; the varying"
 echo ">>>                       coordinate will be the non-displayed horizontal"
 echo ">>>                       coordinate"
 echo ">>>                      use a negative number to make multiple images for"
 echo ">>>                       the constant time no. -<node>, with the"
 echo ">>>                        non-displayed horizontal coordinate varying"
 echo ">>>                        from one frame to another"
 echo ">>>    <first>     first time step no."
 echo ">>>    <last>      last  time step no."
 echo ">>>    <step>      time step between consequtive frames"
 echo ">>>                 (optional, set to  1  if not specified by user)"
 echo ">>>    <delay>     frame delay in ms (optional, set to  10  if not specified by user)"
 echo ">>> "
 echo ">>>  The script will produce a gif animation."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'mlayersection.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Example:"
 echo ">>> ========"
 echo ">>> "
 echo ">>>  mlayersection.sh hycom_expt007.nc . thknss salin lat 30 47 49"
 echo ">>>    will produce an animation for lat-z crossection normal to longitude"
 echo ">>>     node no. 30, of the variable 'salin' on the file 'hycom_expt007.nc',"
 echo ">>>     w/ layers given by 'thknss' on the same file,"
 echo ">>>     from timestep no. 47, 48 and 49"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$8" = "" ]
then

 echo ">>> NOTE: The user is **STRONGLY** recommended to copy"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'mlayersection.sh' is given,"
 echo ">>>  and at least specify"
 echo ">>>   * color map (palette)"
 echo ">>>  -otherwise, the series of files produced by  mlayersection.sh"
 echo ">>>   will (usually) NOT have the same color map, details are provided by executing"
 echo ">>>   mlayersection.sh --help"
 echo ">>> "
 echo ">>> NOTE: This script assumes that the variable to depict has been"
 echo ">>>  stored with dimensions in the order x  -y  -layer(-time)"
 echo ">>>                                   or lon-lat-layer(-time)"
 echo ">>>  (w/ Fortran style sequence of dimensions)."
 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  mlayersection.sh <hfile> <varfile> <hname> <varname> <dimname> <node> <first> <last> (<step>)"
 echo ">>> e.g. (for a meridional crossection)"
 echo ">>>  mlayersection.sh hycom_expt007.nc . thknss salin lat 30 47 49"
 echo ">>> "
 echo ">>> NOTE: use  0  for x-y-z & lon-lat-z fields; the varying coordinate"
 echo ">>>        will be the non-displayed horizontal coordinate"
 echo ">>>       use a negative number to make multiple images for the"
 echo ">>>        constant time no. -<node>, with the non-displayed horizontal"
 echo ">>>        coordinate varying from one frame to another"
 echo ">>> Execute"
 echo ">>>   mlayersection.sh --help"
 echo ">>>  for more details, and an example."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
p2gbin=`which ppmtogif`
animbin=`which gifsicle`
giffiles=""


nclfile=".mlayersection.ncl"
test -r nclfile && rm nclfile


hfile=$1
vfile=$2
hname=$3
vname=$4
dname=$5
node=$6
firsttime=$7
lasttime=$8

value=$9
if [ "$value" = "" ]
then
 step="1"
 delay="10"
else
 if [ $value -le 0 ]
 then
  step="1"
  delay=$((-value))
 else
  step=$value
  shift
  value=$9
 fi
 if [ "$value" = "" ]
 then
  delay="10"
 else
  delay=$((-value))
 fi
fi


# ...set flags:
if [ $firsttime -lt 0 ]
then
 firsttime=$((-firsttime))
 keep_gif="1"
else
 keep_gif="0"
fi
if [ $lasttime -lt 0 ]
then
 lasttime=$((-lasttime))
 keep_ppm="1"
else
 keep_ppm="0"
fi

# ...check sequence:
if [ $firsttime -gt $lasttime ]
then
 echo "Illegal syntax for time stepping. Terminates."
 exit
fi


if [ "$vfile" = "." ]
then
  vfile=$hfile
fi

if [ "$vname" = "." ]
then
  vname=$hname
fi


if [ "$node" -gt 0 ]
then
 if [ "$dname" = "x" ] || [ "$dname" = "lon" ]
 then
  dim0string='(t0,:,'$node',:)'
  dimstring='(t0,:,'$node',x1:x2)'
 else
  dim0string='(t0,:,:,'$node')'
  dimstring='(t0,:,x1:x2,'$node')'
 fi
elif [ "$node" -eq 0 ]
then
 if [ "$dname" = "x" ] || [ "$dname" = "lon" ]
 then
  dim0string='(:,t0,:)'
  dimstring='(:,t0,x1:x2)'
 else
  dim0string='(:,:,t0)'
  dimstring='(:,x1:x2,t0)'
 fi
else
 node=$((-node))
 if [ "$dname" = "x" ] || [ "$dname" = "lon" ]
 then
  dim0string='($node,:,t0,:)'
  dimstring='($node,:,t0,x1:x2)'
 else
  dim0string='($node,:,:,t0)'
  dimstring='($node,:,x1:x2,t0)'
 fi
fi


###  MAIN TIME LOOP STARTS BELOW


time=$firsttime


while true
do


 cat $ncdir/mhead.ncl                                        > $nclfile

 echo 't0 = '$time'-1'                                      >> $nclfile

 echo 'ncfile = addfile("'$hfile'","r")'                    >> $nclfile

 echo 'ncvar0 = ncfile->'$hname$dim0string                  >> $nclfile
 echo 'nxy = dimsizes(ncvar0)'                              >> $nclfile
 echo 'nx  = nxy(1)'                                        >> $nclfile
 echo 'ny  = nxy(0)'                                        >> $nclfile

 if  [ -e userdef.ncl ]
 then
  userfile="userdef.ncl"
 else
  userfile=$ncdir/"userdef.ncl"
 fi
 cat $userfile                                              >> $nclfile
 cat $ncdir/colormap.ncl                                    >> $nclfile

 echo 'x1 = x1-1'                                           >> $nclfile
 echo 'x2 = x2-1'                                           >> $nclfile

 if  [ "$dname" = "lon" ] || [ "$dname" = "lat" ]
 then
  llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  lonname=${llstring:ca:cn}
  llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  latname=${llstring:ca:cn}
  if [ "$dname" = "lon" ]
  then
   altname=$latname
  elif [ "$dname" = "lat" ]
  then
   altname=$lonname
  fi
 fi

 echo 'ncfile2 = addfile("'$vfile'","r")'                   >> $nclfile
 echo ''                                                    >> $nclfile
 echo 'thknss = ncfile->'$hname$dimstring                   >> $nclfile
 echo 'invar  = ncfile2->'$vname$dimstring                  >> $nclfile


 if [ "$dname" = "lon" ]
 then
   echo 'lon = ncfile->'$lonname'(x1:x2)'                   >> $nclfile
   echo 'lat = ncfile->'$latname'(:)'                       >> $nclfile
 elif [ "$dname" = "lat" ]
 then
   echo 'lon = ncfile->'$lonname'(:)'                       >> $nclfile
   echo 'lat = ncfile->'$latname'(x1:x2)'                   >> $nclfile
 fi

 echo 'if(isatt(thknss,"_FillValue"))'                      >> $nclfile
 echo ' thknss@_FillValue = ncfile->'$hname'@_FillValue'    >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' if(isatt(thknss,"missing_value"))'                  >> $nclfile
 echo '  thknss@_FillValue = ncfile->'$hname'@missing_value'>> $nclfile
 echo ' end if'                                             >> $nclfile
 echo 'end if'                                              >> $nclfile
 echo 'if(isatt(invar,"_FillValue"))'                       >> $nclfile
 echo ' invar@_FillValue = ncfile2->'$vname'@_FillValue'    >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' if(isatt(thknss,"missing_value"))'                  >> $nclfile
 echo '  invar@_FillValue = ncfile2->'$vname'@missing_value'>> $nclfile
 echo ' end if'                                             >> $nclfile
 echo 'end if'                                              >> $nclfile

 echo 'if(isatt(thknss,"add_offset"))'                      >> $nclfile
 echo ' addoffh = ncfile->'$hname'@add_offset'              >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' addoffh = 0.'                                       >> $nclfile
 echo 'end if'                                              >> $nclfile
 echo 'if(isatt(invar,"add_offset"))'                       >> $nclfile
 echo ' addoffv = ncfile2->'$vname'@add_offset'             >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' addoffv = 0.'                                       >> $nclfile
 echo 'end if'                                              >> $nclfile

 echo 'if(isatt(thknss,"scale_factor"))'                    >> $nclfile
 echo ' scaleh = ncfile->'$hname'@scale_factor'             >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' scaleh = 1.'                                        >> $nclfile
 echo 'end if'                                              >> $nclfile
 echo 'if(isatt(invar,"scale_factor"))'                     >> $nclfile
 echo ' scalev = ncfile2->'$vname'@scale_factor'            >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' scalev = 1.'                                        >> $nclfile
 echo 'end if'                                              >> $nclfile

 cat $ncdir/llsize.ncl                                      >> $nclfile
 cat $ncdir/mcontour.ncl                                    >> $nclfile
 cat $ncdir/ticklabels.ncl                                  >> $nclfile

 echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
 echo ' title=":F"+fontno+":" + mytitle'                    >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' title=":F"+fontno+":" + invar@long_name'            >> $nclfile
 if [ "$time" != "0" ]
 then
  echo ' title= title  + ", time no. "'                     >> $nclfile
  echo ' title= title  + sprinti("%4.4i", '$time')'         >> $nclfile
 fi
 if [ "$altname" = "lon" ] || [ "$altname" = "lat" ]
 then
  echo ' node = '$node                                      >> $nclfile
  echo ' title= title  + ", '$altname'. "'                  >> $nclfile
  echo ' title= title  + sprintf("%6.2f", '$altname'(node))'>> $nclfile
 fi
 echo 'end if'                                              >> $nclfile
 echo ''                                                    >> $nclfile

 cat $ncdir/layersection.ncl                                >> $nclfile

 if [ "$dname" = "lon" ]
 then
   echo 'if( lon(last) .gt. 360.) then'                     >> $nclfile
   echo '  rsrcs@sfXArray = lon(first:last) - 360.'         >> $nclfile
   echo 'else'                                              >> $nclfile
   echo '  rsrcs@sfXArray = lon(first:last)'                >> $nclfile
   echo 'end if'                                            >> $nclfile
 elif [ "$dname" = "lat" ]
 then
   echo 'rsrcs@sfXArray = lat(first:last)'                  >> $nclfile
 fi

 echo 'rsrcs@sfYArray = -level'                             >> $nclfile
 cat $ncdir/ticklabels.ncl                                  >> $nclfile

 echo '; ***override  mapcont  & reset  ncvar when layer no. is requested***'>> $nclfile
 if [ "$hname" = "$vname" ]
 then
   echo 'delete(rsrcs@cnLevels)'                            >> $nclfile
   echo 'map_cont       = fspan(1.5,ny-0.5,ny-1)'           >> $nclfile
   echo 'rsrcs@cnLevelSelectionMode = "ExplicitLevels"'     >> $nclfile
   echo 'rsrcs@cnLevels = map_cont'                         >> $nclfile
   echo 'layer@_FillValue = 999'                            >> $nclfile
   echo 'do x = first, last'                                >> $nclfile
   echo '  do z = 0, nz-1'                                  >> $nclfile
   echo '    if( layer(z,x) .eq. ny) then'                  >> $nclfile
   echo '      ncvar(z,x) = ncvar@_FillValue'               >> $nclfile
   echo '    else'                                          >> $nclfile
   echo '      ncvar(z,x) = layer(z,x) + 1'                 >> $nclfile
   echo '    end if'                                        >> $nclfile
   echo '  end do'                                          >> $nclfile
   echo 'end do'                                            >> $nclfile
 fi

 echo 'rsrcs@tiMainString    = title'                       >> $nclfile
 echo 'rsrcs@gsnFrame        = False'                       >> $nclfile
 echo ''                                                    >> $nclfile

 if  [ -e usermod.ncl ]
 then
  cat usermod.ncl                                           >> $nclfile
 fi

 echo 'eplot = gsn_csm_contour(ewks,ncvar(:,first:last),rsrcs)' >> $nclfile
 echo ''                                                    >> $nclfile
 echo 'frame(ewks)'                                         >> $nclfile


 if [ "$hname" = "$vname" ]
 then
  echo 'delete(map_cont)'                                   >> $nclfile
 fi
 cat $ncdir/del_mlayersection.ncl                           >> $nclfile


 $nclbin < $nclfile

 echo ""
 echo ""


 rm $nclfile

 if [ $time -gt 999 ]
 then
  ttext=$time
 elif [ $time -gt 99 ]
 then
  ttext="0"$time
 elif [ $time -gt 9 ]
 then
  ttext="00"$time
 else
  ttext="000"$time
 fi


 if [ -r ncvar_e.eps ]
 then

  if [ -r resize ]
  then
   resize=`head -1 resize`
  else
   resize=`head -1 $ncdir/resize`
  fi

  if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$p2gbin" == "" ] || [ "$animbin" == "" ]
  then
   echo "...missing one or more of the binaries"
   echo "    pstopnm  pnmcrop  ppm2gif  gifsicle"
   echo "...terminating w/ no action taken"
   exit
  else
   $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop 'ncvar_e.eps'
   $pnmcropbin 'ncvar_e.eps001.ppm' >  $vname$ttext'.ppm'
   $p2gbin $vname$ttext'.ppm' > $vname$ttext'.gif'
   rm 'ncvar_e.eps' 'ncvar_e.eps001.ppm'
   if [ $keep_ppm -eq 0 ]
   then
    rm $vname$ttext'.ppm'
   fi
   giffiles=$giffiles" "$vname$ttext'.gif'
  fi

 fi

 time=$((time+step))

 if [ $time -gt $lasttime ]
 then
  break
 fi

done


# ...make 4 additional copies of last frame to pause animation briefly
finaltime=$((time+step+step+step))
otext=$ttext
while [ $time -le $finaltime ]
do
 if [ $time -gt 999 ]
 then
  ttext=$time
 elif [ $time -gt 99 ]
 then
  ttext="0"$time
 elif [ $time -gt 9 ]
 then
  ttext="00"$time
 else
  ttext="000"$time
 fi
 cp $vname$otext'.gif' $vname$ttext'.gif'
 giffiles=$giffiles" "$vname$ttext'.gif'
 time=$((time+step))
done

# ...create gif animation
$animbin --loop -O2 -d $delay $giffiles > $vname.igif

if [ $keep_gif -eq 0 ]
then
 /bin/rm $giffiles
fi


exit
