
echo ""
echo "        mvector.sh  /  ncl-metno 1.2"
#echo "          Date:     12 April, 2006"
#echo "          Contact:  a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-12  implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots
#             now producing a gif animation, using gifsicle & more


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>> NOTE: This script will provide a set of output files, where"
 echo ">>>  the value of the final (3rd or 4th) dimension changes from"
 echo ">>>  one output file to the next. Below, we assume that this"
 echo ">>>  dimension is time."
 echo ">>> "
 echo ">>> ...if  <option> is  4d  or  4dmap:"
 echo ">>>  mvector.sh <option> <file> <u> <v> <depth> <first> <last> (<step>) (-<delay>)"
 echo ">>> ...if  <option> is  3d  or  3dmap:"
 echo ">>>  mvector.sh <option> <file> <u> <v> <first> <last> (<step>) (-<delay>)"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [34]d    - [34]D fields"
 echo ">>>                  [34]dmap - [34]D fields, dims. are lon & lat"
 echo ">>>                  ...[34]dmap will be displayed on a geogr. map"
 echo ">>>    <file>      name of the netcdf file"
 echo ">>>    <u>         name of variable w/ velocity in the x-direction"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>    <v>         name of variable w/ velocity in the y-direction"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>    <depth>     vertical level no."
 echo ">>>    <first>     first time step no."
 echo ">>>    <last>      last  time step no."
 echo ">>>    <step>      time step between consequtive frames"
 echo ">>>                 (optional, set to  1  if not specified by user)"
 echo ">>>    <delay>     frame delay in ms (optional, set to  10  if not specified by user)"
 echo ">>> "
 echo ">>>  The script will produce a gif animation."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'mvector.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * vector spec.s (size, distance, curly/regular vectors)"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Example:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  mvector.sh 4dmap hydrography.nc u v 1 5 15 2"
 echo ">>>    will produce vectors on a lon-lat grid w/ a map, of"
 echo ">>>     the first vertical level and time steps no. 5, 7, 9, 11, 13 and 15,"
 echo ">>>     based on variables 'u' and 'v' on the file 'hydrography.nc'"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$6" = "" ]
then

 echo ">>> Syntax error. Correct syntax:"
 echo ">>> ...if  <option> is  4d  or  4dmap:"
 echo ">>>  mvector.sh <option> <file> <u> <v> <depth> <first> <last> (<step>) (-<delay>)"
 echo ">>> ...if  <option> is  3d  or  3dmap:"
 echo ">>>  mvector.sh <option> <file> <u> <v> <first> <last> (<step>) (-<delay>)"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                  [34]d    - [34]D fields"
 echo ">>>                  [34]dmap - [34]D fields, dims. are lon & lat"
 echo ">>>                  ...[34]dmap will be displayed on a geogr. map"
 echo ">>> e.g."
 echo ">>>  mvector.sh 4dmap hydrography.nc u v 1 5 15 2"
 echo ">>> "
 echo ">>>  The script will produce a gif animation."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   mvector.sh --help"
 echo ">>>  for more details, and an example."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
p2gbin=`which ppmtogif`
animbin=`which gifsicle`
giffiles=""


nclfile=".mvector.ncl"
test -r nclfile && rm nclfile

option=$1
ncfile=$2
ncvaru=$3
ncvarv=$4


if [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$7" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'mvector.sh --help'   for details"
  exit
 fi

 level=$5
 firsttime=$6
 lasttime=$7
 value=$8
 if [ "$value" = "" ]
 then
  step="1"
  delay="10"
 else
  if [ $value -le 0 ]
  then
   step="1"
   delay=$((-value))
  else
   step=$value
   value=$9
  fi
  if [ "$value" = "" ]
  then
   delay="10"
  else
   delay=$((-value))
  fi
 fi

 inc_z="yes"

elif [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 firsttime=$5
 lasttime=$6
 value=$7
 if [ "$value" = "" ]
 then
  step="1"
  delay="10"
 else
  if [ $value -le 0 ]
  then
   step="1"
   delay=$((-value))
  else
   step=$value
   value=$8
  fi
  if [ "$value" = "" ]
  then
   delay="10"
  else
   delay=$((-value))
  fi
 fi

 inc_z="no"

else

 echo ">>> Unrecognized <option>, type  'mvector.sh --help'  for details"
 exit

fi


# ...set flags:
if [ $firsttime -lt 0 ]
then
 firsttime=$((-firsttime))
 keep_gif="1"
else
 keep_gif="0"
fi
if [ $lasttime -lt 0 ]
then
 lasttime=$((-lasttime))
 keep_ppm="1"
else
 keep_ppm="0"
fi

# ...check sequence:
if [ $firsttime -gt $lasttime ]
then
 echo "Illegal syntax for time stepping. Terminates."
 exit
fi


if [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc_z" = "no" ]
then
 dim0string='(t0,:,:)'
 dimstring='(t0,y1:y2,x1:x2)'
else
 dim0string='(t0,z0,:,:)'
 dimstring='(t0,z0,y1:y2,x1:x2)'
fi



###  MAIN TIME LOOP STARTS BELOW


time=$firsttime

while true
do


 if [ $time -gt 999 ]
 then
  ttitle='":F"+fontno+":'$ncvaru'/'$ncvarv', timestep no. '$time
 elif [ $time -gt 99 ]
 then
  ttitle='":F"+fontno+":'$ncvaru'/'$ncvarv', timestep no. 0'$time
 elif [ $time -gt 9 ]
 then
  ttitle='":F"+fontno+":'$ncvaru'/'$ncvarv', timestep no. 00'$time
 else
  ttitle='":F"+fontno+":'$ncvaru'/'$ncvarv', timestep no. 000'$time
 fi
 if [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
 then
  title=$ttitle', level no. '$level'"'
 else
  title=$ttitle'"'
 fi
 

 cat $ncdir/mhead.ncl                                        > $nclfile

 echo 'ncfile = addfile("'$ncfile'","r")'                   >> $nclfile

 echo 't0 = '$time'-1'                                      >> $nclfile
 if [ "$inc_z" = "yes" ]
 then
  echo 'z0 = '$level'-1'                                    >> $nclfile
 fi

 echo ''                                                    >> $nclfile
 echo 'ncvar0 = ncfile->'$ncvaru$dim0string                 >> $nclfile
 echo 'nxy = dimsizes(ncvar0)'                              >> $nclfile
 echo 'nx  = nxy(1)'                                        >> $nclfile
 echo 'ny  = nxy(0)'                                        >> $nclfile

 if  [ -e userdef.ncl ]
 then
  userfile="userdef.ncl"
 else
  userfile=$ncdir/"userdef.ncl"
 fi
 cat $userfile                                              >> $nclfile
 cat $ncdir/minicolormap.ncl                                >> $nclfile

 if   [ "$prjctntype" = "xy" ]
 then
  cat $ncdir/limits.ncl                                     >> $nclfile
 else
  llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  lonname=${llstring:ca:cn}
  llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  latname=${llstring:ca:cn}
  cat $ncdir/lllimits.ncl                                   >> $nclfile
 fi

 echo ''                                                    >> $nclfile
 echo 'ncvar = ncfile->'$ncvaru$dimstring                   >> $nclfile
 echo 'ncvarv = ncfile->'$ncvarv$dimstring                  >> $nclfile

 echo 'if(isatt(ncvar,"_FillValue"))'                       >> $nclfile
 echo ' ncvar@_FillValue = ncfile->'$ncvaru'@_FillValue'    >> $nclfile
 echo ' ncvarv@_FillValue = ncfile->'$ncvarv'@_FillValue'   >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' if(isatt(ncvar,"missing_value"))'                   >> $nclfile
 echo '  ncvar@_FillValue = ncfile->'$ncvaru'@missing_value'>> $nclfile
 echo '  ncvarv@_FillValue = ncfile->'$ncvarv'@missing_value'>> $nclfile
 echo ' end if'                                             >> $nclfile
 echo 'end if'                                              >> $nclfile

 echo 'if(isatt(ncvar,"add_offset"))'                       >> $nclfile
 echo ' addoffu = ncfile->'$ncvaru'@add_offset'             >> $nclfile
 echo ' addoffv = ncfile->'$ncvarv'@add_offset'             >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' addoffu = 0.'                                       >> $nclfile
 echo ' addoffv = 0.'                                       >> $nclfile
 echo 'end if'                                              >> $nclfile

 echo 'if(isatt(ncvar,"scale_factor"))'                     >> $nclfile
 echo ' scaleu = ncfile->'$ncvaru'@scale_factor'            >> $nclfile
 echo ' scalev = ncfile->'$ncvarv'@scale_factor'            >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' scaleu = 1.'                                        >> $nclfile
 echo ' scalev = 1.'                                        >> $nclfile
 echo 'end if'                                              >> $nclfile

 if [ "$prjctntype" = "ll" ]
 then
  cat $ncdir/llsize.ncl                                     >> $nclfile
  cat $ncdir/mvector.ncl                                    >> $nclfile
  echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
  echo " lon_min  = ncvar&"$lonname"(0)"                    >> $nclfile
  echo " lon_max  = ncvar&"$lonname"(x2-x1)"                >> $nclfile
  echo " lat_min  = ncvar&"$latname"(0)"                    >> $nclfile
  echo " lat_max  = ncvar&"$latname"(y2-y1)"                >> $nclfile
  echo "else"                                               >> $nclfile
  echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
  echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
  echo " system(\"/bin/rm -f ncvar_e.eps\")"                >> $nclfile
  echo " exit"                                              >> $nclfile
  echo "end if"                                             >> $nclfile
  cat $ncdir/llplot.ncl                                     >> $nclfile
  cat $ncdir/vectorpos.ncl                                  >> $nclfile
 else
  cat $ncdir/size.ncl                                       >> $nclfile
  cat $ncdir/mvector.ncl                                    >> $nclfile
  cat $ncdir/ticklabels.ncl                                 >> $nclfile
 fi

 echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
 echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'    >> $nclfile
 echo 'else'                                                >> $nclfile
 echo ' rsrcs@tiMainString    = '$title                     >> $nclfile
 echo 'end if'                                              >> $nclfile
 echo ''                                                    >> $nclfile

 echo 'rsrcs@gsnFrame        = False'                       >> $nclfile
 echo ''                                                    >> $nclfile
 echo 'varu=scaleu*ncvar +addoffu'                          >> $nclfile
 echo 'varv=scalev*ncvarv+addoffv'                          >> $nclfile

 if  [ -e usermod.ncl ]
 then
  cat usermod.ncl                                           >> $nclfile
 fi

 if [ "$prjctntype" = "ll" ]
 then
  echo 'eplot = gsn_csm_vector_map(ewks,varu,varv,rsrcs)'   >> $nclfile
 else
  echo 'eplot = gsn_csm_vector(ewks,varu,varv,rsrcs)'       >> $nclfile
 fi
 echo ''                                                    >> $nclfile
 echo 'frame(ewks)'                                         >> $nclfile

 cat $ncdir/del_mvector.ncl                                 >> $nclfile


 $nclbin < $nclfile

 echo ""
 echo ""

 rm $nclfile


 if [ $time -gt 999 ]
 then
  ttext=$time
 elif [ $time -gt 99 ]
 then
  ttext="0"$time
 elif [ $time -gt 9 ]
 then
  ttext="00"$time
 else
  ttext="000"$time
 fi

 if [ -r ncvar_e.eps ]
 then

  if [ -r resize ]
  then
   resize=`head -1 resize`
  else
   resize=`head -1 $ncdir/resize`
  fi

  if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$p2gbin" == "" ] || [ "$animbin" == "" ]
  then
   echo "...missing one or more of the binaries"
   echo "    pstopnm  pnmcrop  ppm2gif  gifsicle"
   echo "...terminating w/ no action taken"
   exit
  else
   $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop 'ncvar_e.eps'
   $pnmcropbin 'ncvar_e.eps001.ppm' >  'uv'$ttext'.ppm'
   $p2gbin 'uv'$ttext'.ppm' > 'uv'$ttext'.gif'
   rm 'ncvar_e.eps' 'ncvar_e.eps001.ppm'
   if [ $keep_ppm -eq 0 ]
   then
    rm 'uv'$ttext'.ppm'
   fi
   giffiles=$giffiles" uv"$ttext'.gif'
  fi

 fi


 time=$((time+step))
 if [ $time -gt $lasttime ]
 then
  break
 fi


done


# ...make 4 additional copies of last frame to pause animation briefly
finaltime=$((time+step+step+step))
otext=$ttext
while [ $time -le $finaltime ]
do
 if [ $time -gt 999 ]
 then
  ttext=$time
 elif [ $time -gt 99 ]
 then
  ttext="0"$time
 elif [ $time -gt 9 ]
 then
  ttext="00"$time
 else
  ttext="000"$time
 fi
 cp 'uv'$otext'.gif' 'uv'$ttext'.gif'
 giffiles=$giffiles" uv"$ttext'.gif'
 time=$((time+step))
done

# ...create gif animation
$animbin --loop -O2 -d $delay $giffiles > uv.igif

if [ $keep_gif -eq 0 ]
then
 /bin/rm $giffiles
fi


exit
