
echo ""
echo "        section.sh   /  ncl-metno 1.2"
#echo "          Date:      5 April, 2006"
#echo "          Contact:   a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-05  now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  section.sh <file> <variable> <ndims> <dim1> <dim2> <node1> (<node2>)"
 echo ">>> where"
 echo ">>>    <file>      name of netCDF file"
 echo ">>>    <variable>  name of variable to depict"
 echo ">>>    <ndims>     no. of dimensions of the variable (3 or 4; for 2, use contour.sh xy ...)"
 echo ">>>    <dim1>      crossection's 1. dimension no. (1-3) [Fortran style]"
 echo ">>>    <dim2>      crossection's 2. dimension no. (2-4) [Fortran style]"
 echo ">>>    <node1>     node no. of first  non-depicted dimension"
 echo ">>>                 a negative <node1> value is interpreted as a flag that"
 echo ">>>                  stops the ncl script from being deleted"
 echo ">>>    <node2>     node no. of second non-depicted dimension (if <ndims> is 4)"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'section.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Example:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  section.sh sst.nc sst 3 1 3 60"
 echo ">>>    if the first and second dimensions are longitude and"
 echo ">>>     latitude, and the third is time, this will produce"
 echo ">>>     a Hovmoller diagram of sst variability along latitude"
 echo ">>>     node no. 60, based on results on the file 'sst.nc'"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$6" = "" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  section.sh <file> <variable> <ndims> <dim1> <dim2> <node1> (<node2>)"
 echo ">>> where"
 echo ">>>    <file>      name of netCDF file"
 echo ">>>    <variable>  name of variable to depict"
 echo ">>>    <ndims>     no. of dimensions of the variable (3 or 4; for 2, use contour.sh xy ...)"
 echo ">>>    <dim1>      crossection's 1. dimension no. (1-3) [Fortran style]"
 echo ">>>    <dim2>      crossection's 2. dimension no. (2-4) [Fortran style]"
 echo ">>>    <node1>     node no. of first  non-depicted dimension"
 echo ">>>                 a negative <node1> value is interpreted as a flag that"
 echo ">>>                  stops the ncl script from being deleted"
 echo ">>>    <node2>     node no. of second non-depicted dimension (if <ndims> is 4)"
 echo ">>> e.g. (for a Hovmoller diagram)"
 echo ">>>  section.sh sst.nc sst 3 1 3 60"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   section.sh --help"
 echo ">>>  for more details, and an example."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".section.ncl"
test -r nclfile && rm nclfile

ncfile=$1
ncvar=$2
ndims=$3

dim1=$4
dim1no=$((ndims-dim1))

dim2=$5
dim2no=$((ndims-dim2))


pdim3value=$6
if [ $pdim3value -gt 0 ]
then
 dim3value=$pdim3value
else
 dim3value=$((0-pdim3value))
fi
dim3val0=$((dim3value-1))

if [ $ndims -ne 3 ]
then
  pdim4value=$7
 if [ $pdim4value -gt 0 ]
 then
  dim4value=$pdim4value
 else
  dim4value=$((0-pdim4value))
 fi
 dim4val0=$((dim4value-1))
else
  pdim4value=$pdim3value
fi


# Store in sequence:
if [ $dim2no -gt $dim1no ]
then
  ncdim1no=$dim1no
  ncdim2no=$dim2no
  xdim=$dim2
  ydim=$dim1
else
  ncdim1no=$dim2no
  ncdim2no=$dim1no
  xdim=$dim1
  ydim=$dim2
fi


if [ $ndims -ne 3 ]
then

  if [ $ncdim2no -eq 1 ]
  then
    dim0string='(:,:,'$dim4val0','$dim3val0')'
    dimstring='(y1:y2,x1:x2,'$dim4val0','$dim3val0')'
  elif [ $ncdim1no -eq 2 ]
  then
    dim0string='('$dim4val0','$dim3val0',:,:)'
    dimstring='('$dim4val0','$dim3val0',y1:y2,x1:x2)'
  elif [ $ncdim1no -eq 0 ]
  then
    if [ $ncdim2no -eq 2 ]
    then
      dim0string='(:,'$dim4val0',:,'$dim3val0')'
      dimstring='(y1:y2,'$dim4val0',x1:x2,'$dim3val0')'
    else
      dim0string='(:,'$dim4val0','$dim3val0',:)'
      dimstring='(y1:y2,'$dim4val0','$dim3val0',x1:x2)'
    fi
  else
    if [ $ncdim2no -eq 2 ]
    then
      dim0string='('$dim4val0',:,:,'$dim3val0')'
      dimstring='('$dim4val0',y1:y2,x1:x2,'$dim3val0')'
    else
      dim0string='('$dim4val0',:,'$dim3val0',:)'
      dimstring='('$dim4val0',y1:y2,'$dim3val0',x1:x2)'
    fi
  fi

else

  if [ $ncdim1no -eq 1 ]
  then
    dim0string='('$dim3val0',:,:)'
    dimstring='('$dim3val0',y1:y2,x1:x2)'
  elif [ $ncdim2no -eq 1 ]
  then
    dim0string='(:,:,'$dim3val0')'
    dimstring='(y1:y2,x1:x2,'$dim3val0')'
  else
    dim0string='(:,'$dim3val0',:)'
    dimstring='(y1:y2,'$dim3val0',x1:x2)'
  fi

fi

title='":F"+fontno+":'$ncvar', x-dim. is no. '$xdim', y-dim. is no. '$ydim'"'


cat $ncdir/head.ncl                                         > $nclfile

echo 'ncfile = addfile("'$ncfile'","r")'                   >> $nclfile
echo ''                                                    >> $nclfile
echo 'ncvar0 = ncfile->'$ncvar$dim0string                  >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                              >> $nclfile
echo 'nx  = nxy(1)'                                        >> $nclfile
echo 'ny  = nxy(0)'                                        >> $nclfile


if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                              >> $nclfile
cat $ncdir/colormap.ncl                                    >> $nclfile
cat $ncdir/limits.ncl                                      >> $nclfile

echo ''                                                    >> $nclfile
echo 'ncvar  = ncfile->'$ncvar$dimstring                   >> $nclfile

echo 'if(isatt(ncvar,"_FillValue"))'                       >> $nclfile
echo ' ncvar@_FillValue = ncfile->'$ncvar'@_FillValue'     >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(ncvar,"missing_value"))'                   >> $nclfile
echo '  ncvar@_FillValue = ncfile->'$ncvar'@missing_value' >> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"add_offset"))'                       >> $nclfile
echo ' addoff = ncfile->'$ncvar'@add_offset'               >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoff = 0.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"scale_factor"))'                     >> $nclfile
echo ' scale = ncfile->'$ncvar'@scale_factor'              >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scale = 1.'                                         >> $nclfile
echo 'end if'                                              >> $nclfile


echo 'width  = maxsize'                                    >> $nclfile
echo 'height = maxsize'                                    >> $nclfile
cat $ncdir/contour.ncl                                     >> $nclfile
cat $ncdir/ticklabels.ncl                                  >> $nclfile

echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' rsrcs@tiMainString    = '$title                     >> $nclfile
echo 'end if'                                              >> $nclfile
echo ''                                                    >> $nclfile

echo 'var=scale*ncvar+addoff'                              >> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                           >> $nclfile
fi

echo 'xplot = gsn_csm_contour(xwks,var,rsrcs)'             >> $nclfile
echo 'eplot = gsn_csm_contour(ewks,var,rsrcs)'             >> $nclfile
echo ''                                                    >> $nclfile
echo 'end'                                                 >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pdim3value -gt 0 ] && [ $pdim4value -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps $ncvar'.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop $ncvar'.eps'
  $pnmcropbin $ncvar'.eps001.ppm' >  $ncvar'.ppm'
  $convertbin $ncvar'.ppm' -resize $resize $ncvar'.png'
  rm $ncvar'.eps001.ppm' $ncvar'.ppm'
 fi

fi


exit
