
echo ""
echo "           slice.sh  /  ncl-metno 1.2"
#echo "          Date:      11 April, 2006"
#echo "          Contact:   a.melsom@met.no"
echo ""

# 2005-11-21  based on layersection.sh, v. 1.1
# 2006-04-11  included option to use 2d fields for lon's & lat's;
#             implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots;
#             now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> NOTE: This script assumes that the variable to depict has been"
 echo ">>>  stored with dimensions in the order x  -y  -layer(-time)"
 echo ">>>                                   or lon-lat-layer(-time)"
 echo ">>>  (w/ Fortran style sequence of dimensions)."
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  slice.sh (layer) <option> <hfile> <varfile> <hname> <varname> <depth> (<time>)"
 echo ">>> where"
 echo ">>>    (layer)     include an initial argument 'layer' if you want isopleths"
 echo ">>>                 to be drawn for layer interfaces, ignore otherwise"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [34]d    - [34]D fields"
 echo ">>>                  [34]dmap - [34]D fields, dims. are lon & lat"
 echo ">>>                  [34]dMap - [34]D fields, lon & lat are 2d fields"
 echo ">>>                  ...[34]d[mM]ap will be displayed on a geogr. map"
 echo ">>>    <hfile>     name of netCDF file w/thickness results"
 echo ">>>    <varfile>   name of netCDF file w/requested variable"
 echo ">>>                 you may use '.' if both variables are on the same file"
 echo ">>>    <hname>     name of thickness variable"
 echo ">>>    <varname>   name of variable to depict"
 echo ">>>             NOTE! This script requires the existence of an"
 echo ">>>                   attribute 'missing_value' to <varname>"
 echo ">>>    <depth>     depth for which the contour plot is to be drawn"
 echo ">>> (positive integer, 1-4 digits)"
 echo ">>>    <time>      time step no. to depict"
 echo ">>>                a negative <time> value is interpreted as a flag that"
 echo ">>>                 stops the ncl script from being deleted"
 echo ">>> "
 echo ">>>  Special case:"
 echo ">>>   If <option> is one of [34]dMap, the name of the 2d longitude and"
 echo ">>>    latitude variable may be specified on the command line:"
 echo ">>>   slice.sh (layer) <option> <lonname> <latname> <hfile> <varfile> <hname> <varname> <depth> (<time>)"
 echo ">>>    (Alternatively, if these names are not 'lon' or 'lat', 'userdef.ncl' may"
 echo ">>>     be edited when option is one of [34]dMap.)"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'slice.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * names of longitude & latitude variables"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Example:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  slice.sh 4dmap hycom_expt007.nc . thknss salin 1000 30"
 echo ">>>    will produce a depiction on a lon-lat grid w/ a map,"
 echo ">>>     of the variable 'salin' on the file 'hycom_expt007.nc',"
 echo ">>>     for a depth of 1000m w/ layers given by 'thknss' on the"
 echo ">>>     same file, from timestep no. 30"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi


# Should contour lines for layers be added?
arg=$1
if [ "$arg" = "layer" ] || [ "$arg" = "Layer" ] || [ "$arg" = "LAYER" ]
then
 layers_on="1"
 shift
 arg=$1
else
 layers_on="0"
fi



if [ "$6" = "" ]
then

 echo ">>> "
 echo ">>> NOTE: This script assumes that the variable to depict has been"
 echo ">>>  stored with dimensions in the order x  -y  -layer(-time)"
 echo ">>>                                   or lon-lat-layer(-time)"
 echo ">>>  (w/ Fortran style sequence of dimensions)."
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  slice.sh (layer) <option> <hfile> <varfile> <hname> <varname>\\"
 echo ">>>                                              <depth> (<time>)" 
 echo ">>> e.g. (for a horizontal crossection at 1000m)"
 echo ">>>  slice.sh 4dmap hycom_expt007.nc . thknss salin 1000 30"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   slice.sh --help"
 echo ">>>  for more details, and an example."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".slice.ncl"
test -r nclfile && rm nclfile


option=$arg
hfile=$2
if [ "$option" = "4dMap" ] || [ "$option" = "3dMap" ] || [ "$option" = "2dMap" ]
then
 dotpos=`expr index "$hfile" .`
 if [ $dotpos -eq 0 ]
 then
  lonname=$hfile
  shift
  latname=$2
  shift
  hfile=$2
 else
  lonname=""
  latname=""
 fi
else
 lonname=""
 latname=""
fi
vfile=$3
hname=$4
vname=$5
pdepth=$6
ptime=$7


if [ "$vfile" = "." ]
then
  vfile=$hfile
else
  vfile=$vfile
fi

if [ "$vname" = "." ]
then
  vname=$hname
fi

# reset to act as flag?
# ...depth level:
if [ $pdepth -gt 0 ]
then
  depth=$pdepth
else
  depth=$((0-pdepth))
fi
# ...time step no.:
if [ "$ptime" = "" ]
then
  time="none"
elif [ $ptime -gt 0 ]
then
  time=$ptime
else
  time=$((0-ptime))
  pdepth=$((0-depth))
fi


if   [ "$option" = "3dMap" ] || [ "$option" = "4dMap" ]
then
 if [ $layers_on -eq 0 ]
 then
  prjctntype="LL"
 else
  echo ""
  echo "WARNING!"
  echo "...don't know how to overlay a map on a map,"
  echo "   resorting to Cartesian coordinates (i.e., no map)"
  echo ""
  prjctntype="xy"
 fi
elif [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 if [ $layers_on -eq 0 ]
 then
  prjctntype="ll"
 else
  echo ""
  echo "WARNING!"
  echo "...don't know how to overlay a map on a map,"
  echo "   resorting to Cartesian coordinates (i.e., no map)"
  echo ""
  prjctntype="xy"
 fi
else
 prjctntype="xy"
fi

if [ "$time" = "none" ]
then
  dim0string='(:,:,:)'
  dimstring='(:,y1:y2,x1:x2)'
else
  dim0string='('$time',:,:,:)'
  dimstring='('$time',:,y1:y2,x1:x2)'
fi


cat $ncdir/head.ncl                                         > $nclfile

echo 'ncfile = addfile("'$hfile'","r")'                    >> $nclfile

echo 'ncvar0 = ncfile->'$hname$dim0string                  >> $nclfile
echo 'nxyz = dimsizes(ncvar0)'                             >> $nclfile
echo 'nx   = nxyz(2)'                                      >> $nclfile
echo 'ny   = nxyz(1)'                                      >> $nclfile
echo 'nnz  = nxyz(0)'                                      >> $nclfile

echo 'depth  = '$depth                                     >> $nclfile


if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                              >> $nclfile
cat $ncdir/colormap.ncl                                    >> $nclfile

if   [ "$prjctntype" = "xy" ]
then
 cat $ncdir/limits.ncl                                     >> $nclfile
else
 if [ "$lonname" = "" ]
 then
  llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  lonname=${llstring:ca:cn}
 fi
 if [ "$latname" = "" ]
 then
  llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
  ca=`expr index "$llstring" '"'`
  cn=`expr index "${llstring:$ca+1}" '"'`
  latname=${llstring:ca:cn}
 fi
cat $ncdir/lllimits.ncl                                   >> $nclfile
fi


echo 'ncfile2 = addfile("'$vfile'","r")'                   >> $nclfile
echo ''                                                    >> $nclfile
echo 'thknss = ncfile->'$hname$dimstring                   >> $nclfile
echo 'invar  = ncfile2->'$vname$dimstring                  >> $nclfile


echo 'if(isatt(thknss,"_FillValue"))'                      >> $nclfile
echo ' thknss@_FillValue = ncfile->'$hname'@_FillValue'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(thknss,"missing_value"))'                  >> $nclfile
echo '  thknss@_FillValue = ncfile->'$hname'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(invar,"_FillValue"))'                       >> $nclfile
echo ' invar@_FillValue = ncfile2->'$vname'@_FillValue'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(thknss,"missing_value"))'                  >> $nclfile
echo '  invar@_FillValue = ncfile2->'$vname'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(thknss,"add_offset"))'                      >> $nclfile
echo ' addoffh = ncfile->'$hname'@add_offset'              >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoffh = 0.'                                       >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(invar,"add_offset"))'                       >> $nclfile
echo ' addoffv = ncfile2->'$vname'@add_offset'             >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoffv = 0.'                                       >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(thknss,"scale_factor"))'                    >> $nclfile
echo ' scaleh = ncfile->'$hname'@scale_factor'             >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scaleh = 1.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile
echo 'if(isatt(invar,"scale_factor"))'                     >> $nclfile
echo ' scalev = ncfile2->'$vname'@scale_factor'            >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scalev = 1.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile

cat $ncdir/slice.ncl                                       >> $nclfile

if   [ "$prjctntype" = "LL" ]
then
 cat $ncdir/llsize.ncl                                     >> $nclfile
 cat $ncdir/contour.ncl                                    >> $nclfile
 echo "if (isfilevarcoord(ncfile,\""$hname"\",\""$latname"\") .or. isfilevarcoord(ncfile,\""$hname"\",\""$lonname"\")) then" >> $nclfile
 echo " print(\"..."$lonname" and/or "$latname" are not 2D fields\")" >> $nclfile
 echo " print(\"   which they must be for the Map options, but\")" >> $nclfile
 echo " print(\"   one or both are (1D) variable coordinate(s)\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"                >> $nclfile
 echo " exit"                                              >> $nclfile
 echo "else"                                               >> $nclfile
 echo " if (isfilevar(ncfile,\""$latname"\") .and. isfilevar(ncfile,\""$lonname"\")) then" >> $nclfile
 echo "  lon2d  = ncfile->"$lonname"(y1:y2,x1:x2)"         >> $nclfile
 echo "  lat2d  = ncfile->"$latname"(y1:y2,x1:x2)"         >> $nclfile
 cat $ncdir/mod_lon.ncl                                    >> $nclfile
 echo " else"                                              >> $nclfile
 echo "  print(\"...could not find a proper set of map variables,\")" >> $nclfile
 echo "  print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo "  system(\"/bin/rm -f ncvar_e.eps\")"               >> $nclfile
 echo "  exit"                                             >> $nclfile
 echo " end if"                                            >> $nclfile
 echo "end if"                                             >> $nclfile
 cat $ncdir/LLplot.ncl                                     >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 cat $ncdir/llsize.ncl                                     >> $nclfile
 cat $ncdir/contour.ncl                                    >> $nclfile
 echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
 echo " lon_min  = ncvar&"$lonname"(0)"                    >> $nclfile
 echo " lon_max  = ncvar&"$lonname"(x2-x1)"                >> $nclfile
 echo " lat_min  = ncvar&"$latname"(0)"                    >> $nclfile
 echo " lat_max  = ncvar&"$latname"(y2-y1)"                >> $nclfile
 echo "else"                                               >> $nclfile
 echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
 echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"                >> $nclfile
 echo " exit"                                              >> $nclfile
 echo "end if"                                             >> $nclfile
 cat $ncdir/llplot.ncl                                     >> $nclfile
else
 cat $ncdir/size.ncl                                       >> $nclfile
 cat $ncdir/contour.ncl                                    >> $nclfile
 cat $ncdir/ticklabels.ncl                                 >> $nclfile
fi

echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
echo ' title=":F"+fontno+":" + mytitle'                    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' title=":F"+fontno+":" + invar@long_name'            >> $nclfile
echo ' title= title  + ", depth "'                         >> $nclfile
echo ' title= title  + sprinti("%4.4i", '$depth')'         >> $nclfile
if [ "$time" != "none" ]
then
 echo ' title= title  + ", time no. "'                     >> $nclfile
 echo ' title= title  + sprinti("%4.4i", '$time')'         >> $nclfile
fi
echo 'end if'                                              >> $nclfile
echo ''                                                    >> $nclfile



echo '; ***override  mapcont  & reset  ncvar when layer no. is requested***'>> $nclfile
if [ "$hname" = "$vname" ]
then
  cat $ncdir/slice_override.ncl                            >> $nclfile
fi
if [ $layers_on -eq 1 ]
then
  cat $ncdir/slice_layers_on.ncl                           >> $nclfile
fi

echo 'rsrcs@tiMainString    = title'                       >> $nclfile
echo ''                                                    >> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                           >> $nclfile
fi

if [ $layers_on -eq 1 ]
then
 echo 'rsrcs@gsnFrame        = False'                      >> $nclfile
fi

if   [ "$prjctntype" = "LL" ]
then
 echo 'ncvar@lon2d = lon2d'                                >> $nclfile
 echo 'ncvar@lat2d = lat2d'                                >> $nclfile
 echo 'xplot = gsn_csm_contour_map(xwks,ncvar,rsrcs)'      >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,ncvar,rsrcs)'      >> $nclfile
elif [ "$prjctntype" = "ll" ]
then
 echo 'xplot = gsn_csm_contour_map(xwks,ncvar,rsrcs)'      >> $nclfile
 echo 'eplot = gsn_csm_contour_map(ewks,ncvar,rsrcs)'      >> $nclfile
else
 echo 'xplot = gsn_csm_contour(xwks,ncvar,rsrcs)'          >> $nclfile
 echo 'eplot = gsn_csm_contour(ewks,ncvar,rsrcs)'          >> $nclfile
fi
echo ''                                                    >> $nclfile

if [ $layers_on -eq 1 ]
then
 cat $ncdir/slicelines.ncl                                 >> $nclfile
fi

echo 'end'                                                 >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pdepth -ge 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps $vname'.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop $vname'.eps'
  $pnmcropbin $vname'.eps001.ppm' >  $vname'.ppm'
  $convertbin $vname'.ppm' -resize $resize $vname'.png'
  rm $vname'.eps001.ppm' $vname'.ppm'
 fi

fi


exit
