
;  --------------------------------------------------------------
;  Specify the title as  mytitle  and the font as  fontno
;   ...affects all scripts
;   ...if   mytitle  is set to "auto", the individual script will
;       produce a title
;  --------------------------------------------------------------

mytitle= "auto"
;fontno=07         ;complex   -roman
;fontno=09         ;complex   -script
;fontno=10         ;complex   -italic
;fontno=11         ;duplex    -roman
;fontno=13         ;triplex   -roman
;fontno=21         ;helvetica
fontno=22         ;helvetica -bold
;fontno=25         ;times     -roman
;fontno=26         ;times     -bold
;fontno=29         ;courier


;  --------------------------------------------------
;  Alter grid spec. to zoom in on subdomain
;   ...y1, ny  does not apply to *layersection.sh
;  Special case: if x2<x1 for a map plot, the numbers
;                will be interpreted as min & max
;                longitudes (x) and latitudes (y)
;  --------------------------------------------------

x1=  1      ; Leftmost  grid point to depict, for dimension  x  or  lon
x2= nx      ; Rightmost grid point to depict
y1=  1      ; Lowermost grid point to depict, for dimension  y  or  lat
y2= ny      ; Uppermost grid point to depict


;  -------------
;  Set plot size
;  -------------

maxsize = 0.65  ; Max. plot height/width (0-1 ratio)
xp      = 0.1   ; Upper left corner of plot, x-coordinate (0=left  )
yp      = 0.9   ; Upper left corner of plot, y-coordinate (0=bottom)


;  --------------------------------------------------------------
;  Alter contouring spec.
;   ...applies to addlayers.sh, *contour.sh, cmask.sh, v-on-c.sh,
;                 *layersection.sh, section.sh, transport.sh
;  --------------------------------------------------------------

v1=  0.     ; Low  value for isopleths, disregarded when  nv  is 0 or 1
v2=  0.     ; High value for isopleths, disregarded when  nv  is 0 or 1
nv=  0      ; No. of isopleths, there will be  nv+1  colors
            ; NOTE! A negative value of  nv  will trigger use of a
            ;       stretched color interval (geometric), cases:
            ;        nv>-1000: w_1=1, w_2=2, w_3=3, ..., w_n=n
            ;  -1000>nv>-2000: w_1=1, w_2=2, w_3=3, ..., w_n=w_n-1+w_n-2
            ;  -2000>nv      : w_1=1, w_2=2, w_3=3, ..., w_n=w_n-1+w_n-3
            ;       i.e.
            ;        nv>-1000: moderate     streching
            ;  -1000>nv>-2000: large        stretcing
            ;  -2000>nv      : intermediate stretcing
            ; NOTE! In the present version, isopleths are not drawn,
            ;       but spaces between isopleths are filled with various colors


;  --------------------------------------------------
;  Alter vector spec.
;   ...applies to v-on-c.sh, *vector.sh, transport.sh
;  --------------------------------------------------

vsz= 0.05   ; Size (length) of reference vector
vsp= 0.03   ; Speed         of reference vector
vd = 0.015  ; Distance between vectors
curly_on= 1 ; =1: Use curly vectors, otherwise, use standard vectors
            ;
            ; i.e.: The length of vectors will be increased when either the
            ;        vsz  is increased, or the  vsp  is decreased
            ;       Fewer vectors are displayed when  vd  is increased


;  -------------------------------------------------------------
;  Choose color map,
;   set color for masked regions, 
;   set method for cell filling during contouring,
;   ...applies to addlayers.sh *contour.sh, cmask.sh, v-on-c.sh,
;                 *layersection.sh, section.sh, transport.sh
;  -------------------------------------------------------------

mapname= "nrl_sirkes"              ; NRL,            nv <= 17
;mapname= "LR BkBlAqGrYeOrReViWh200" ; rainbow style,  nv <= 17
;mapname= "HR BkBlAqGrYeOrReViWh200"; rainbow style,  nv <= 35
;mapname= "LR BlWhRe"               ; blue/white/red, nv <= 17
;mapname= "HR BlWhRe"               ; blue/white/red, nv <= 35
;mapname= "gsdtol"                  ; grayscale,      nv <= 17
;mapname= "default"                 ; tigerstripes,   cyclic for nv >17

fillcolor= (/0.6, 0.6, 0.6/)        ; color for masking: (/red, green, blue/)

filling= "AreaFill"                 ; fills between computed isopleths
;filling= "CellFill"


;  -------------------------------------------------------------------
;  Alter bottom depth (will be set to largest depth when  bottom = 0 )
;    and no. of vertical z-levels for interpolation  (nz)
;   ...applies to *layersection.sh
;  -------------------------------------------------------------------

bottom =  0.
nz     = 50


;  ---------------------------------------------------------
;   >>> The remaining selections apply ONLY to      <<<
;   >>> command line options  2dmap, 3dmap, 4dmap,  <<<
;   >>>                       2dMap, 3dMap, 4dMap   <<<
;  (which are relevant for most of the provided scripts)
;  ---------------------------------------------------------


;  ------------------------------------------------------------------
;  Provide name of latitude, longitude variables on input netCDF file
;   NOTE: case matters!
;  ------------------------------------------------------------------

lonname="lon"
latname="lat"


;  ------------------------------------------------------------------
;  Limit map boundaries, note that it is unnecessary to take action
;   here if you want to depict the entire domain within boundaries
;   implied by the spec. for x1/x2/y1/y2 above
;  ------------------------------------------------------------------

minlon=-360.
maxlon= 360.
minlat= -90.
maxlat=  90.


;  ---------------------------------------------------------------
;  Alter map projection by commenting the default (top) option and
;                        uncommenting the projection of choice
;  ---------------------------------------------------------------

map= "LambertEqualArea"
;map= "Stereographic"
;map= "Mercator"
;map= "CylindricalEquidistant"
;map= "Orthographic"
;map= "AzimuthalEquidistant"
;map= "Satellite"
;map= "Mollweide"


;  ------------------------------------------------------------------
;  Alter plot routine for coastlines and other map features
;   ...applies to *contour.sh, cmask.sh, v-on-c.sh, *layersection.sh,
;                 transport.sh
;  ------------------------------------------------------------------

coastline = "Ncarg4_1"      ; Intermediate level of map details
;coastline = "Ncarg4_0"      ; Coarse level of map details
;coastline = "RANGS_GSHHS"   ; Advanced level of map details
