
echo ""
echo "        v-on-c.sh  /  ncl-metno 1.2"
#echo "          Date:    11 April, 2006"
#echo "          Contact: a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-05  implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots;
#             now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  v-on-c.sh <option> <uvfile> <varfile> <u> <v> <var> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  ...[234]dmap will be displayed on a geogr. map"
 echo ">>>    <uvfile>    name of the netcdf file w/ <u> and <v>"
 echo ">>>    <varfile>   name of the netcdf file w/ <var>"
 echo ">>>                 you may type '.' if <uvfile> & <varfile> are the same"
 echo ">>>    <u>         name of variable w/ velocity in the x-direction"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>    <v>         name of variable w/ velocity in the y-direction"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>    <var>       name of variable to base filled contours on"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>             NOTE! This script requires the same dimensions for"
 echo ">>>                   <u>, <v> and <var>, i.e., they must all be"
 echo ">>>                   2d, or 2dmap, etc."
 echo ">>>    <d3node>  node no. of third dimension"
 echo ">>>                if <option> is one of 2d, 2dmap and a fourth"
 echo ">>>                  argument is present, or if <d3node> is negative,"
 echo ">>>                  this will be interpreted as a flag that will cause"
 echo ">>>                  the ncl script to remain (see examples below)"
 echo ">>>    <d4node>  node no. of fourth dimension"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'v-on-c.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * vector spec.s (size, distance, curly/regular vectors)"
 echo ">>>   * color map (palette)"
 echo ">>>   * no. of colors"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * names of longitude & latitude variables"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Examples:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  v-on-c.sh 4dmap uv.nc hydrography.nc u v temp 1 10"
 echo ">>>    will produce vectors on top of filled contours for"
 echo ">>>     temperature, on a lon-lat grid w/ a map, of the first node"
 echo ">>>     in the third dimension (usually the top vertical level)"
 echo ">>>     and the tenth node in the fourth dimension (ususally time"
 echo ">>>     step no. 10) based on variables"
 echo ">>>     'u' and 'v' on the file 'uv.nc', and"
 echo ">>>     'temp'      on          'hydrography.nc'"
 echo ">>>  v-on-c.sh 3d surface.nc . u-vel v-vel ssh -1"
 echo ">>>    will produce vectors on top of filled contours for"
 echo ">>>     sea surface height, on a x-y grid of the first node in"
 echo ">>>     the third dimension, based on variables 'u-vel', 'v-vel'"
 echo ">>>     and 'ssh' on the file 'surface.nc'; and the ncl script"
 echo ">>>     will be retained"
 echo ">>>  v-on-c.sh 2dmap ave.nc topography.nc ubaro vbaro topo a"
 echo ">>>    will produce vectors on top of filled contours for"
 echo ">>>     the bottom topography, on a lon-lat grid w/ a map,"
 echo ">>>     based on variables"
 echo ">>>     'ubaro' and 'vbaro' on the file 'ave.nc', and"
 echo ">>>     'topo'              on          'topography.nc';"
 echo ">>>     the ncl script will be retained"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$6" = "" ]
then

 echo ">>> Syntax error. Correct syntax:"
 echo ">>>  v-on-c.sh <option> <uvfile> <varfile> <u> <v> <var> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  ...[234]dmap will be displayed on a geogr. map"
 echo ">>> e.g."
 echo ">>>  v-on-c.sh 4dmap uv.nc hydrography.nc u v temp 1 10"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   v-on-c.sh --help"
 echo ">>>  for more details, and examples."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".v-on-c.ncl"
test -r nclfile && rm nclfile

option=$1
ncfileuv=$2
if [ "$3" = "." ]
then
 ncfilep=$ncfileuv
else
 ncfilep=$3
fi
ncvaru=$4
ncvarv=$5
ncvarp=$6


if [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$8" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'v-on-c.sh --help'   for details"
  exit
 fi

 pd3=$7
 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-pd3))
 fi

 pd4=$8
 if [ $pd4 -gt 0 ]
 then
   d4=$pd4
 else
   d4=$((0-pd4))
 fi

 title='":F"+fontno+":'$ncvaru'/'$ncvarv'/'$ncvarp', 3rd dim. no. '$d3', 4th dim. no. '$d4'"'
 inc3="yes"
 inc4="yes"

elif [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 if [ "$7" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'v-on-c.sh --help'   for details"
  exit
 fi

 pd3=$7

 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-pd3))
 fi

 title='":F"+fontno+":'$ncvaru'/'$ncvarv'/'$ncvarp', 3rd dim. no. '$d3'"'

 pd4=$pd3
 inc3="yes"
 inc4="no"

elif [ "$option" = "2dmap" ] || [ "$option" = "2d" ]
then

 title='":F"+fontno+":'$ncvaru'/'$ncvarv'/'$ncvarp'"'

 if [ "$7" = "" ]
 then
  pd3="1"
  pd4="1"
 else
  pd3="0"
  pd4="0"
 fi
 inc3="no"
 inc4="no"

else

 echo ">>> Unrecognized <option>, type  'v-on-c.sh --help'  for details"
 exit

fi

if [ "$option" = "2dmap" ] || [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc4" = "no" ] && [ "$inc3" = "no" ]
then
 dim0string='(:,:)'
 dimstring='(y1:y2,x1:x2)'
elif [ "$inc4" = "no" ]
then
 dim0string='(d30,:,:)'
 dimstring='(d30,y1:y2,x1:x2)'
else
 dim0string='(d40,d30,:,:)'
 dimstring='(d40,d30,y1:y2,x1:x2)'
fi


cat $ncdir/head.ncl                                           > $nclfile

echo 'ncfile   = addfile("'$ncfilep'","r")'                  >> $nclfile
echo 'ncfileuv = addfile("'$ncfileuv'","r")'                 >> $nclfile

if [ "$inc3" = "yes" ]
then
 echo 'd30 = '$d3'-1'                                        >> $nclfile
fi
if [ "$inc4" = "yes" ]
then
 echo 'd40 = '$d4'-1'                                        >> $nclfile
fi

echo ''                                                      >> $nclfile
echo 'ncvar0 = ncfile->'$ncvarp$dim0string                   >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                                >> $nclfile
echo 'nx  = nxy(1)'                                          >> $nclfile
echo 'ny  = nxy(0)'                                          >> $nclfile

if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                                >> $nclfile
cat $ncdir/colormap.ncl                                      >> $nclfile

if   [ "$prjctntype" = "xy" ]
then
 cat $ncdir/limits.ncl                                       >> $nclfile
else
 llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
 ca=`expr index "$llstring" '"'`
 cn=`expr index "${llstring:$ca+1}" '"'`
 lonname=${llstring:ca:cn}
 llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
 ca=`expr index "$llstring" '"'`
 cn=`expr index "${llstring:$ca+1}" '"'`
 latname=${llstring:ca:cn}
 cat $ncdir/lllimits.ncl                                     >> $nclfile
fi

echo ''                                                      >> $nclfile
echo 'ncvaru = ncfileuv->'$ncvaru$dimstring                  >> $nclfile
echo 'ncvarv = ncfileuv->'$ncvarv$dimstring                  >> $nclfile
echo 'ncvar  = ncfile->'$ncvarp$dimstring                    >> $nclfile

echo ''                                                      >> $nclfile
echo 'if(isatt(ncvaru,"_FillValue"))'                        >> $nclfile
echo ' ncvaru@_FillValue = ncfileuv->'$ncvaru'@_FillValue'   >> $nclfile
echo ' ncvarv@_FillValue = ncfileuv->'$ncvarv'@_FillValue'   >> $nclfile
echo 'else'                                                  >> $nclfile
echo ' if(isatt(ncvaru,"missing_value"))'                    >> $nclfile
echo '  ncvaru@_FillValue = ncfileuv->'$ncvaru'@missing_value'>> $nclfile
echo '  ncvarv@_FillValue = ncfileuv->'$ncvarv'@missing_value'>> $nclfile
echo ' end if'                                               >> $nclfile
echo 'end if'                                                >> $nclfile
echo 'if(isatt(ncvar,"_FillValue"))'                         >> $nclfile
echo ' ncvar@_FillValue = ncfile->'$ncvarp'@_FillValue'      >> $nclfile
echo 'else'                                                  >> $nclfile
echo ' if(isatt(ncvar,"missing_value"))'                     >> $nclfile
echo '  ncvar@_FillValue = ncfile->'$ncvarp'@missing_value'  >> $nclfile
echo ' end if'                                               >> $nclfile
echo 'end if'                                                >> $nclfile

echo ''                                                      >> $nclfile
echo 'if(isatt(ncvaru,"add_offset"))'                        >> $nclfile
echo ' addoffu = ncfileuv->'$ncvaru'@add_offset'             >> $nclfile
echo ' addoffv = ncfileuv->'$ncvarv'@add_offset'             >> $nclfile
echo 'else'                                                  >> $nclfile
echo ' addoffu = 0.'                                         >> $nclfile
echo ' addoffv = 0.'                                         >> $nclfile
echo 'end if'                                                >> $nclfile
echo 'if(isatt(ncvar,"add_offset"))'                         >> $nclfile
echo ' addoff = ncfile->'$ncvarp'@add_offset'                >> $nclfile
echo 'else'                                                  >> $nclfile
echo ' addoff = 0.'                                          >> $nclfile
echo 'end if'                                                >> $nclfile

echo ''                                                      >> $nclfile
echo 'if(isatt(ncvaru,"scale_factor"))'                      >> $nclfile
echo ' scaleu = ncfileuv->'$ncvaru'@scale_factor'            >> $nclfile
echo ' scalev = ncfileuv->'$ncvarv'@scale_factor'            >> $nclfile
echo 'else'                                                  >> $nclfile
echo ' scaleu = 1.'                                          >> $nclfile
echo ' scalev = 1.'                                          >> $nclfile
echo 'end if'                                                >> $nclfile
echo 'if(isatt(ncvar,"scale_factor"))'                       >> $nclfile
echo ' scale = ncfile->'$ncvarp'@scale_factor'               >> $nclfile
echo 'else'                                                  >> $nclfile
echo ' scale = 1.'                                           >> $nclfile
echo 'end if'                                                >> $nclfile

if [ "$prjctntype" = "ll" ]
then
 cat $ncdir/llsize.ncl                                       >> $nclfile
 cat $ncdir/v-on-c.ncl                                       >> $nclfile
 echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
 echo " lon_min  = ncvar&"$lonname"(0)"                      >> $nclfile
 echo " lon_max  = ncvar&"$lonname"(x2-x1)"                  >> $nclfile
 echo " lat_min  = ncvar&"$latname"(0)"                      >> $nclfile
 echo " lat_max  = ncvar&"$latname"(y2-y1)"                  >> $nclfile
 echo "else"                                                 >> $nclfile
 echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
 echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"                  >> $nclfile
 echo " exit"                                                >> $nclfile
 echo "end if"                                               >> $nclfile
 cat $ncdir/llplot.ncl                                       >> $nclfile
 cat $ncdir/contourpos.ncl                                   >> $nclfile
 cat $ncdir/vectorpos.ncl                                    >> $nclfile
else
 cat $ncdir/size.ncl                                         >> $nclfile
 cat $ncdir/v-on-c.ncl                                       >> $nclfile
 cat $ncdir/ticklabels.ncl                                   >> $nclfile
fi

echo 'if (mytitle .ne. "auto" ) then'                        >> $nclfile
echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'      >> $nclfile
echo 'else'                                                  >> $nclfile
echo ' rsrcs@tiMainString    = '$title                       >> $nclfile
echo 'end if'                                                >> $nclfile
echo ''                                                      >> $nclfile

echo 'var =scale *ncvar +addoff'                             >> $nclfile
echo 'varu=scaleu*ncvaru+addoffu'                            >> $nclfile
echo 'varv=scalev*ncvarv+addoffv'                            >> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                             >> $nclfile
fi

if [ "$prjctntype" = "ll" ]
then
 echo 'xplot = gsn_csm_vector_scalar_map(xwks,varu,varv,var,rsrcs)' >> $nclfile
 echo 'eplot = gsn_csm_vector_scalar_map(ewks,varu,varv,var,rsrcs)' >> $nclfile
else
 echo 'xplot = gsn_csm_vector_scalar(xwks,varu,varv,var,rsrcs)'     >> $nclfile
 echo 'eplot = gsn_csm_vector_scalar(ewks,varu,varv,var,rsrcs)'     >> $nclfile
fi
echo ''                                                      >> $nclfile
echo 'end'                                                   >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pd3 -gt 0 ] && [ $pd4 -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps 'v-on-c.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop 'v-on-c.eps'
  $pnmcropbin 'v-on-c.eps001.ppm' >  'v-on-c.ppm'
  $convertbin 'v-on-c.ppm' -resize $resize 'v-on-c.png'
  rm 'v-on-c.eps001.ppm' 'v-on-c.ppm'
 fi

fi



exit
