
echo ""
echo "        vector.sh  /  ncl-metno 1.2"
#echo "          Date:     11 April, 2006"
#echo "          Contact:  a.melsom@met.no"
echo ""

# 2005-07-08  included optional final user modifications (as usermod.ncl)
# 2005-11-13  dry values primarily flagged by attribute _FillValue,
#                              alternately by attribute missing_value
# 2006-04-11  implemented zooming with min/max lon's & lat's specified
#                              in  userdef.ncl , for map plots;
#             now producing resized cropped png images using pstoppm & convert


# Check syntax, provide user info.:

if [ "$1" = "--help" ] || [ "$1" = "help" ] || [ "$1" = "-h" ] || [ "$1" = "h" ]
then

 echo ">>> "
 echo ">>> "
 echo ">>> Syntax:"
 echo ">>> ======="
 echo ">>> "
 echo ">>>  vector.sh <option> <file> <u> <v> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  ...[234]dmap will be displayed on a geogr. map"
 echo ">>>    <file>      name of the netcdf file"
 echo ">>>    <u>         name of variable w/ velocity in the x-direction"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>    <v>         name of variable w/ velocity in the y-direction"
 echo ">>>                 on the netcdf file (case sensitive)"
 echo ">>>    <d3node>  node no. of third dimension"
 echo ">>>                if <option> is one of 2d, 2dmap and a fourth"
 echo ">>>                  argument is present, or if <d3node> is negative,"
 echo ">>>                  this will be interpreted as a flag that will cause"
 echo ">>>                  the ncl script to remain (see examples below)"
 echo ">>>    <d4node>  node no. of fourth dimension"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> "
 echo ">>> User specifications:"
 echo ">>> ===================="
 echo ">>> "
 echo ">>>  By copying the default spec.s from"
 echo ">>>    "$ncdir"/userdef.ncl"
 echo ">>>  to the directory where the command 'vector.sh' is given,"
 echo ">>>  the user may specify"
 echo ">>>   * title"
 echo ">>>   * font"
 echo ">>>   * zooming"
 echo ">>>   * vector spec.s (size, distance, curly/regular vectors)"
 echo ">>>   * plot size limits"
 echo ">>>   for geographical maps:"
 echo ">>>   * names of longitude & latitude variables"
 echo ">>>   * map projection"
 echo ">>>   * coastline detail level"
 echo ">>>   (look up, or copy, this file to edit your own 'userdef' file)."
 echo ">>> "
 echo ">>> "
 echo ">>> Examples:"
 echo ">>> ========="
 echo ">>> "
 echo ">>>  vector.sh 4dmap hydrography.nc u v 1 10"
 echo ">>>    will produce vectors on a lon-lat grid w/ a map, for"
 echo ">>>     the first node in the third dimension (usually the top"
 echo ">>>     vertical level) and the tenth node in the fourth"
 echo ">>>     dimension (usually time step no. 10), based on"
 echo ">>>     variables 'u' and 'v' on the file 'hydrography.nc'"
 echo ">>>  vector.sh 3d surface.nc u-vel v-vel -1"
 echo ">>>    will produce vectors on a x-y grid of the first node"
 echo ">>>     in the third dimension, based on variables 'u-vel' and"
 echo ">>>     'v-vel' on the file 'surface.nc'; and the ncl-script"
 echo ">>>      will be retained"
 echo ">>>  vector.sh 2dmap topography.nc ubaro vbaro a"
 echo ">>>    will produce vectors on a lon-lat grid w/ a map,"
 echo ">>>    based on variables 'ubaro' and vbaro' on the file"
 echo ">>>    'topography.nc'; and the ncl-script will be retained"
 echo ">>> "
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

if [ "$4" = "" ]
then

 echo ">>> Syntax error. Correct syntax:"
 echo ">>>  vector.sh <option> <file> <u> <v> [<d3node> (<d4node>)]"
 echo ">>> where"
 echo ">>>    <option>    specifies dimensions and geo- or nongeo-grid"
 echo ">>>                 implemented:"
 echo ">>>                  [234]d    - [234]D fields"
 echo ">>>                  [234]dmap - [234]D fields, dims. are lon & lat"
 echo ">>>                  ...[234]dmap will be displayed on a geogr. map"
 echo ">>> e.g."
 echo ">>>  vector.sh 4dmap hydrography.nc u v 1 10"
 echo ">>> "
 echo ">>>  The script will produce an eps-file and a png-file."
 echo ">>> "
 echo ">>> Execute"
 echo ">>>   vector.sh --help"
 echo ">>>  for more details, and examples."
 echo ">>> "
 echo ">>> Terminating."
 echo ""

 exit
fi

nclbin=`which ncl`
pstopnmbin=`which pstopnm`
pnmcropbin=`which pnmcrop`
convertbin=`which convert`


nclfile=".vector.ncl"
test -r nclfile && rm nclfile

option=$1
ncfile=$2
ncvaru=$3
ncvarv=$4


if [ "$option" = "4dmap" ] || [ "$option" = "4d" ]
then

 if [ "$6" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'vector.sh --help'   for details"
  exit
 fi

 pd3=$5
 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-pd3))
 fi

 pd4=$6
 if [ $pd4 -gt 0 ]
 then
   d4=$pd4
 else
   d4=$((0-pd4))
 fi

 title='":F"+fontno+":'$ncvaru'/'$ncvarv', 3rd dim. no. '$d3', 4th dim. no. '$d4'"'

 inc3="yes"
 inc4="yes"

elif [ "$option" = "3dmap" ] || [ "$option" = "3d" ]
then

 if [ "$5" = "" ]
 then
  echo ">>> Not enough command line arguments, type"
  echo ">>>   'vector.sh --help'   for details"
  exit
 fi

 pd3=$5
 if [ $pd3 -gt 0 ]
 then
   d3=$pd3
 else
   d3=$((0-pd3))
 fi

 title='":F"+fontno+":'$ncvaru'/'$ncvarv', 3rd dim. no. '$d3'"'

 pd4=$pd3
 inc3="yes"
 inc4="no"

elif [ "$option" = "2dmap" ] || [ "$option" = "2d" ]
then

 title='":F"+fontno+":'$ncvaru'/'$ncvarv'"'

 if [ "$5" = "" ]
 then
  pd3="1"
  pd4="1"
 else
  pd3="0"
  pd3="0"
 fi
 inc3="no"
 inc4="no"

else

 echo ">>> Unrecognized <option>, type  'vector.sh --help'  for details"
 exit

fi

if [ "$option" = "2dmap" ] || [ "$option" = "3dmap" ] || [ "$option" = "4dmap" ]
then
 prjctntype="ll"
else
 prjctntype="xy"
fi

if   [ "$inc4" = "no" ] && [ "$inc3" = "no" ]
then
 dim0string='(:,:)'
 dimstring='(y1:y2,x1:x2)'
elif [ "$inc4" = "no" ]
then
 dim0string='(d30,:,:)'
 dimstring='(d30,y1:y2,x1:x2)'
else
 dim0string='(d40,d30,:,:)'
 dimstring='(d40,d30,y1:y2,x1:x2)'
fi


cat $ncdir/head.ncl                                         > $nclfile

echo 'ncfile = addfile("'$ncfile'","r")'                   >> $nclfile

if [ "$inc3" = "yes" ]
then
 echo 'd30 = '$d3'-1'                                      >> $nclfile
fi
if [ "$inc4" = "yes" ]
then
 echo 'd40 = '$d4'-1'                                      >> $nclfile
fi

echo ''                                                    >> $nclfile
echo 'ncvar0 = ncfile->'$ncvaru$dim0string                 >> $nclfile
echo 'nxy = dimsizes(ncvar0)'                              >> $nclfile
echo 'nx  = nxy(1)'                                        >> $nclfile
echo 'ny  = nxy(0)'                                        >> $nclfile

if  [ -e userdef.ncl ]
then
 userfile="userdef.ncl"
else
 userfile=$ncdir/"userdef.ncl"
fi
cat $userfile                                              >> $nclfile
cat $ncdir/minicolormap.ncl                                >> $nclfile

if   [ "$prjctntype" = "xy" ]
then
 cat $ncdir/limits.ncl                                     >> $nclfile
else
 llstring=`grep 'lonname' $userfile | grep -v ';lonname' | grep -v '; lonname' | head -1`
 ca=`expr index "$llstring" '"'`
 cn=`expr index "${llstring:$ca+1}" '"'`
 lonname=${llstring:ca:cn}
 llstring=`grep 'latname' $userfile | grep -v ';latname' | grep -v '; latname' | head -1`
 ca=`expr index "$llstring" '"'`
 cn=`expr index "${llstring:$ca+1}" '"'`
 latname=${llstring:ca:cn}
 cat $ncdir/lllimits.ncl                                   >> $nclfile
fi

echo ''                                                    >> $nclfile
echo 'ncvar  = ncfile->'$ncvaru$dimstring                  >> $nclfile
echo 'ncvarv = ncfile->'$ncvarv$dimstring                  >> $nclfile

echo 'if(isatt(ncvar,"_FillValue"))'                       >> $nclfile
echo ' ncvar@_FillValue = ncfile->'$ncvaru'@_FillValue'    >> $nclfile
echo ' ncvarv@_FillValue = ncfile->'$ncvarv'@_FillValue'   >> $nclfile
echo 'else'                                                >> $nclfile
echo ' if(isatt(ncvar,"missing_value"))'                   >> $nclfile
echo '  ncvar@_FillValue = ncfile->'$ncvaru'@missing_value'>> $nclfile
echo '  ncvarv@_FillValue = ncfile->'$ncvarv'@missing_value'>> $nclfile
echo ' end if'                                             >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"add_offset"))'                       >> $nclfile
echo ' addoffu = ncfile->'$ncvaru'@add_offset'             >> $nclfile
echo ' addoffv = ncfile->'$ncvarv'@add_offset'             >> $nclfile
echo 'else'                                                >> $nclfile
echo ' addoffu = 0.'                                       >> $nclfile
echo ' addoffv = 0.'                                       >> $nclfile
echo 'end if'                                              >> $nclfile

echo 'if(isatt(ncvar,"scale_factor"))'                     >> $nclfile
echo ' scaleu = ncfile->'$ncvaru'@scale_factor'            >> $nclfile
echo ' scalev = ncfile->'$ncvarv'@scale_factor'            >> $nclfile
echo 'else'                                                >> $nclfile
echo ' scaleu = 1.'                                        >> $nclfile
echo ' scalev = 1.'                                        >> $nclfile
echo 'end if'                                              >> $nclfile

if [ "$prjctntype" = "ll" ]
then
 cat $ncdir/llsize.ncl                                     >> $nclfile
 cat $ncdir/vector.ncl                                     >> $nclfile
 echo "if (all(iscoord(ncvar,(/\""$latname"\",\""$lonname"\"/)))) then" >> $nclfile
 echo " lon_min  = ncvar&"$lonname"(0)"                    >> $nclfile
 echo " lon_max  = ncvar&"$lonname"(x2-x1)"                >> $nclfile
 echo " lat_min  = ncvar&"$latname"(0)"                    >> $nclfile
 echo " lat_max  = ncvar&"$latname"(y2-y1)"                >> $nclfile
 echo "else"                                               >> $nclfile
 echo " print(\"...could not find a proper set of dimensional variables,\")" >> $nclfile
 echo " print(\"...tried "$lonname", "$latname" from "$userfile"\")" >> $nclfile
 echo " system(\"/bin/rm -f ncvar_e.eps\")"                >> $nclfile
 echo " exit"                                              >> $nclfile
 echo "end if"                                             >> $nclfile
 cat $ncdir/llplot.ncl                                     >> $nclfile
 cat $ncdir/vectorpos.ncl                                  >> $nclfile
else
 cat $ncdir/size.ncl                                       >> $nclfile
 cat $ncdir/vector.ncl                                     >> $nclfile
 cat $ncdir/ticklabels.ncl                                 >> $nclfile
fi

echo 'if (mytitle .ne. "auto" ) then'                      >> $nclfile
echo ' rsrcs@tiMainString    = ":F"+fontno+":"+mytitle'    >> $nclfile
echo 'else'                                                >> $nclfile
echo ' rsrcs@tiMainString    = '$title                     >> $nclfile
echo 'end if'                                              >> $nclfile
echo ''                                                    >> $nclfile

echo 'varu=scaleu*ncvar +addoffu'                          >> $nclfile
echo 'varv=scalev*ncvarv+addoffv'                          >> $nclfile

if  [ -e usermod.ncl ]
then
 cat usermod.ncl                                      >> $nclfile
fi

if [ "$prjctntype" = "ll" ]
then
 echo 'xplot = gsn_csm_vector_map(xwks,varu,varv,rsrcs)'   >> $nclfile
 echo 'eplot = gsn_csm_vector_map(ewks,varu,varv,rsrcs)'   >> $nclfile
else
 echo 'xplot = gsn_csm_vector(xwks,varu,varv,rsrcs)'       >> $nclfile
 echo 'eplot = gsn_csm_vector(ewks,varu,varv,rsrcs)'       >> $nclfile
fi
echo ''                                                    >> $nclfile
echo 'end'                                                 >> $nclfile


$nclbin < $nclfile

echo ''


if [ $pd3 -gt 0 ] && [ $pd4 -gt 0 ]
then
 rm $nclfile
else
 echo 'Saving ncl script file as '$nclfile
 echo ''
fi


if [ -r ncvar_e.eps ]
then

 if [ -r resize ]
 then
  resize=`head -1 resize`
 else
  resize=`head -1 $ncdir/resize`
 fi

 mv ncvar_e.eps 'uv.eps'
 if [ "$pstopnmbin" == "" ] || [ "$pnmcropbin" == "" ] || [ "$convertbin" == "" ]
 then
  echo "...missing one or more of the binaries"
  echo "    pstopnm  pnmcrop  convert"
  echo "...no  png  file has been produced"
 else
  $pstopnmbin -xborder 0 -yborder 0 -portrait -nocrop 'uv.eps'
  $pnmcropbin 'uv.eps001.ppm' >  'uv.ppm'
  $convertbin 'uv.ppm' -resize $resize 'uv.png'
  rm 'uv.eps001.ppm' 'uv.ppm'
 fi

fi


exit
